# Copyright 2020 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for error classes."""

from ccx_messaging.error import CCXMessagingError


input_msg = {
    "topic": "topic name",
    "partition": "partition name",
    "offset": 1234,
    "url": "any/url",
    "identity": {"identity": {"internal": {"org_id": "12345678"}}},
    "timestamp": "2020-01-23T16:15:59.478901889Z",
    "cluster_name": "clusterName",
}


def test_error_formatting():
    """Test the error formatting."""
    err = CCXMessagingError("CCXMessagingError")
    assert err is not None

    fmt = err.format(input_msg)
    expected = "Status: Error; Topic: topic name; Cause: CCXMessagingError"
    assert fmt == expected


def test_error_without_additional_data():
    """Test CCXMessagingError without additional_data (backwards compatibility)."""
    err = CCXMessagingError("Test error message")

    assert err is not None
    assert str(err) == "Test error message"
    assert err.additional_data is None


def test_error_with_additional_data():
    """Test CCXMessagingError with additional_data parameter."""
    test_data = {"archive_path": "/tmp/test.tar", "cluster_id": "test-cluster"}
    err = CCXMessagingError("Test error with data", additional_data=test_data)

    assert err is not None
    assert str(err) == "Test error with data"
    assert err.additional_data == test_data
    assert err.additional_data["archive_path"] == "/tmp/test.tar"
    assert err.additional_data["cluster_id"] == "test-cluster"


def test_error_with_empty_additional_data():
    """Test CCXMessagingError with empty dict as additional_data."""
    err = CCXMessagingError("Test error", additional_data={})

    assert err is not None
    assert str(err) == "Test error"
    assert err.additional_data == {}



# Copyright 2020, 2021, 2022 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `ConsumerWatcher` class."""

import time
from tarfile import TarFile
from unittest.mock import MagicMock, patch

import pytest
from insights.core.archives import TarExtractor

from ccx_messaging.watchers.stats_watcher import (
    StatsWatcher,
    ARCHIVE_TYPE_LABEL,
    ARCHIVE_TYPE_VALUES,
)


_INVALID_PORTS = [None, "8000", 8000.0, 70000]


@pytest.fixture(params=ARCHIVE_TYPE_VALUES)
def label_value(request):
    """Set the label value for the running test."""
    return request.param


@pytest.mark.parametrize("value", _INVALID_PORTS)
def test_stats_watcher_initialize_invalid_port(value):
    """Test passing invalid data types or values to the `StatsWatcher` initializer fails."""
    with pytest.raises((TypeError, PermissionError, OverflowError, OSError)):
        _ = StatsWatcher(value)


_VALID_PORTS = [{}, {"prometheus_port": 9500}, {"prometheus_port": 80}]


@pytest.mark.parametrize("value", _VALID_PORTS)
@patch("ccx_messaging.watchers.stats_watcher.start_http_server")
def test_stats_watcher_initialize(start_http_server_mock, value):
    """Test valid values in the initialize `StatsWatcher`."""
    StatsWatcher(**value)
    port = value.get("prometheus_port", 8000)  # 8000 is the default value
    start_http_server_mock.assert_called_with(port)


def check_initial_metrics_state(w):
    """Check that all metrics are initialized."""
    for value in ARCHIVE_TYPE_VALUES:
        assert w._recv_total._value.get() == 0
        assert w._filtered_total._value.get() == 0
        assert w._extracted_total.labels(**{ARCHIVE_TYPE_LABEL: value})._value.get() == 0
        assert w._processed_total.labels(**{ARCHIVE_TYPE_LABEL: value})._value.get() == 0
        assert w._processed_timeout_total.labels(**{ARCHIVE_TYPE_LABEL: value})._value.get() == 0
        assert w._published_total.labels(**{ARCHIVE_TYPE_LABEL: value})._value.get() == 0
        assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: value})._value.get() == 0
        assert w._not_handling_total._value.get() == 0
        # Check initial values of histogram metrics
        assert w._downloaded_total._sum.get() == 0
        assert w._download_duration._sum.get() == 0
        assert w._process_duration.labels(**{ARCHIVE_TYPE_LABEL: value})._sum.get() == 0
        assert w._publish_duration.labels(**{ARCHIVE_TYPE_LABEL: value})._sum.get() == 0


def init_timestamps(w):
    """Initialize all timestamps in watcher."""
    t = time.time()
    w._start_time = t
    w._downloaded_time = t
    w._processed_time = t
    w._published_time = t


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_stats_watcher_on_recv(label_value):
    """Test the on_recv() method."""
    input_msg = {"identity": {}}

    # construct watcher object
    w = StatsWatcher(prometheus_port=8001)
    init_timestamps(w)
    w._archive_metadata["type"] = label_value

    # check that all metrics are initialized
    check_initial_metrics_state(w)

    # change metrics
    w.on_recv(input_msg)

    # test new metrics values
    assert w._recv_total._value.get() == 1
    assert w._filtered_total._value.get() == 0
    assert len(w._downloaded_total._labelvalues) == 0
    assert w._processed_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._processed_timeout_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._published_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._not_handling_total._value.get() == 0


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_stats_watcher_on_filter(label_value):
    """Test the on_filter() method."""
    # construct watcher object
    w = StatsWatcher(prometheus_port=8001)
    init_timestamps(w)
    w._archive_metadata["type"] = label_value

    # check that all metrics are initialized
    check_initial_metrics_state(w)

    # change metrics
    w.on_filter()

    # test new metrics values
    assert w._recv_total._value.get() == 0
    assert w._filtered_total._value.get() == 1
    assert w._downloaded_total._sum.get() == 0
    assert w._processed_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._processed_timeout_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._published_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._not_handling_total._value.get() == 0


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_stats_watcher_on_download(label_value):
    """Test the on_download() method."""
    # construct watcher object
    w = StatsWatcher(prometheus_port=8002)
    init_timestamps(w)
    w._archive_metadata["type"] = label_value

    # check that all metrics are initialized
    check_initial_metrics_state(w)

    # change metrics
    with patch("ccx_messaging.watchers.stats_watcher.os.path.getsize", lambda x: 100):
        w.on_download("path")

    # test new metrics values
    assert w._recv_total._value.get() == 0
    assert w._filtered_total._value.get() == 0
    assert w._downloaded_total._sum.get() == 100
    assert w._processed_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._processed_timeout_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._published_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._not_handling_total._value.get() == 0

    assert w._archive_metadata["size"] == 100


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_stats_watcher_on_process(label_value):
    """Test the on_process() method."""
    input_msg = {"identity": {}}

    # construct watcher object
    w = StatsWatcher(prometheus_port=8003)
    init_timestamps(w)
    w._archive_metadata["type"] = label_value

    # check that all metrics are initialized
    check_initial_metrics_state(w)

    # change metrics
    w.on_process(input_msg, "{result}")

    # test new metrics values
    assert w._recv_total._value.get() == 0
    assert w._filtered_total._value.get() == 0
    assert w._downloaded_total._sum.get() == 0
    assert w._processed_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 1
    assert w._processed_timeout_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._published_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._not_handling_total._value.get() == 0


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_stats_watcher_on_process_timeout(label_value):
    """Test the on_process_timeout() method."""
    # construct watcher object
    w = StatsWatcher(prometheus_port=8004)
    init_timestamps(w)
    w._archive_metadata["type"] = label_value

    # change metrics
    w.on_process_timeout()

    # test new metrics values
    assert w._recv_total._value.get() == 0
    assert w._filtered_total._value.get() == 0
    assert w._downloaded_total._sum.get() == 0
    assert w._processed_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._processed_timeout_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 1
    assert w._published_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._not_handling_total._value.get() == 0


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_stats_watcher_on_consumer_success(label_value):
    """Test the on_consumer_success() method."""
    input_msg = {"identity": {}}

    # construct watcher object
    w = StatsWatcher(prometheus_port=8005)
    init_timestamps(w)
    w._archive_metadata["type"] = label_value

    # change metrics
    w.on_consumer_success(input_msg, "broker", "{result}")

    # test new metrics values
    assert w._recv_total._value.get() == 0
    assert w._filtered_total._value.get() == 0
    assert w._downloaded_total._sum.get() == 0
    assert w._processed_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._processed_timeout_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._published_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 1
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._not_handling_total._value.get() == 0


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_stats_watcher_on_consumer_failure(label_value):
    """Test the on_consumer_failure() method."""
    input_msg = {"identity": {}}

    # construct watcher object
    w = StatsWatcher(prometheus_port=8006)
    init_timestamps(w)
    w._archive_metadata["type"] = label_value

    # change metrics
    w.on_consumer_failure(input_msg, Exception("something"))

    # test new metrics values
    assert w._recv_total._value.get() == 0
    assert w._filtered_total._value.get() == 0
    assert w._downloaded_total._sum.get() == 0
    assert w._processed_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._processed_timeout_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._published_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 1
    assert w._not_handling_total._value.get() == 0

    # reset downloaded time
    w._downloaded_time = None

    # change metrics again
    w.on_consumer_failure(input_msg, Exception("something"))

    # metric should change
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 2

    # reset processed time
    w._processed_time = None

    # change metrics again
    w.on_consumer_failure(input_msg, Exception("something"))

    # metric should change again
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 3

    # now try this - downloaded time is not None and processed time is none
    w._downloaded_time = time.time()
    w._processed_time = None

    # change metrics again
    w.on_consumer_failure(input_msg, Exception("something"))

    # metric should change again
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 4


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_stats_watcher_on_not_handled(label_value):
    """Test the on_not_handled() method."""
    input_msg = {"identity": {}}

    # construct watcher object
    w = StatsWatcher(prometheus_port=8007)
    init_timestamps(w)
    w._archive_metadata["type"] = label_value

    # change metrics
    w.on_not_handled(input_msg)

    # test new metrics values
    assert w._recv_total._value.get() == 0
    assert w._filtered_total._value.get() == 0
    assert w._downloaded_total._sum.get() == 0
    assert w._processed_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._processed_timeout_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._published_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._failures_total.labels(**{ARCHIVE_TYPE_LABEL: label_value})._value.get() == 0
    assert w._not_handling_total._value.get() == 1


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_reset_times():
    """Test the method _reset_times()."""
    # construct watcher object
    w = StatsWatcher(prometheus_port=8008)
    init_timestamps(w)

    assert w._start_time is not None
    assert w._downloaded_time is not None
    assert w._processed_time is not None
    assert w._published_time is not None

    w._reset_times()

    assert w._start_time is not None
    assert w._downloaded_time is None
    assert w._processed_time is None
    assert w._published_time is None


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
def test_reset_archive_metadata(label_value):
    """Test the method _reset_times()."""
    # construct watcher object
    w = StatsWatcher(prometheus_port=8009)
    w._archive_metadata["type"] = label_value

    w._reset_archive_metadata()

    assert w._archive_metadata["type"] == "ocp"


ON_EXTRACT_WITH_EXTRACTOR_IDENTIFIERS = [
    ("ols", [True, False]),
    ("hypershift", [False, True]),
    ("ocp", [False, False]),
]


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
@pytest.mark.parametrize("type_, exists_side_effect", ON_EXTRACT_WITH_EXTRACTOR_IDENTIFIERS)
def test_on_extract_with_extractor(type_, exists_side_effect):
    """Test the method on_extract."""
    # construct watcher object
    w = StatsWatcher(prometheus_port=8009)

    extraction_mock = MagicMock(spec=TarExtractor)
    extraction_mock.tmp_dir = ""

    with patch("os.path.exists") as exists_mock:
        exists_mock.side_effect = exists_side_effect

        w.on_extract(None, None, extraction_mock)
        assert w._archive_metadata["type"] == type_


ON_EXTRACT_WITH_TARFILE_IDENTIFIERS = [
    ("ols", ["openshift_lightspeed.json"]),
    ("hypershift", ["config/id", "config/infrastructure.json"]),
    ("ocp", ["config/id"]),
]


@patch("ccx_messaging.watchers.stats_watcher.start_http_server", lambda *args: None)
@pytest.mark.parametrize("type_, file_content", ON_EXTRACT_WITH_TARFILE_IDENTIFIERS)
def test_on_extract_with_tarfile(type_, file_content):
    """Test the method on_extract."""
    # construct watcher object
    w = StatsWatcher(prometheus_port=8009)

    extraction_mock = MagicMock(spec=TarFile)
    extraction_mock.getnames.return_value = file_content
    w.on_extract(None, None, extraction_mock)
    assert w._archive_metadata["type"] == type_



# Copyright 2023 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Sentry SDK configuration and utility functions."""

import logging
import os

import sentry_sdk
from sentry_sdk.integrations.logging import LoggingIntegration


def get_event_level():
    """Get level of events to monitor (errors only, or error and warnings)."""
    if os.environ.get("SENTRY_CATCH_WARNINGS", False):
        return logging.WARNING
    return logging.ERROR


def init_sentry(dsn=None, transport=None, environment=None, enabled=False):
    """Configure and initialize sentry SDK for this project."""
    if enabled:
        if dsn:
            logging.getLogger(__name__).info("Initializing sentry")
            sentry_logging = LoggingIntegration(level=logging.INFO, event_level=get_event_level())

            sentry_sdk.init(
                dsn=dsn,
                ca_certs="/etc/pki/tls/certs/ca-bundle.crt",
                integrations=[sentry_logging],
                max_breadcrumbs=15,
                transport=transport,
                environment=environment,
            )
        else:
            logger = logging.getLogger(__name__)
            logger.warning("Configuration Warning: Sentry is enabled, but no DSN was provided.")



# Copyright 2023 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `utils/sentry.py` file."""

import logging
import os

from ccx_messaging.utils.sentry import get_event_level


def test_get_event_level():
    """Verify the `get_event_level` function works.

    Check that it returns `logging.WARNING` if SENTRY_CATCH_WARNINGS
    is set, otherwise return `logging.ERROR`.
    """
    os.environ["SENTRY_CATCH_WARNINGS"] = "1"
    assert get_event_level() == logging.WARNING
    os.environ["SENTRY_CATCH_WARNINGS"] = ""
    assert get_event_level() == logging.ERROR



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may naot use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for the MultiplexorPublisher class."""

import json
import gzip
from unittest.mock import call, MagicMock

from ccx_messaging.publishers.multiplexor_kafka_publisher import MultiplexorPublisher


BEST_COMPRESSION = 9
INPUT_MSG = {
    "path": "target/path",
    "cluster_id": "05ef94c8-9468-405e-93cc-bef1e79cde96",
    "sqs_message_id": "AQEBt8CEyiFnyVTY+B2kuJ+gOpCcMsev",
}


def timeStampMasking(message):
    """Mask four bytes in Gzip stream that contain timestamp."""
    message = list(message)
    message[4] = 0
    message[5] = 0
    message[6] = 0
    message[7] = 0
    message = bytes(message)
    return message


def test_init():
    """Check that init creates a valid object."""
    kakfa_config = {
        "bootstrap.servers": "kafka:9092",
    }
    MultiplexorPublisher(
        outgoing_topics={
            "OLS": "ols-topic",
            "DEFAULT": "io-topic",
        },
        **kakfa_config,
    )


def test_normal_produce():
    """Check if message is not gzipped if compression is disabled."""
    topic = "io-topic"
    kakfa_config = {"bootstrap.servers": "kafka:9092"}
    pub = MultiplexorPublisher(**kakfa_config)

    pub.producer = MagicMock()
    pub.produce(json.dumps(INPUT_MSG).strip().encode(), topic)
    pub.producer.produce.assert_called_with(topic, json.dumps(INPUT_MSG).strip().encode())


def test_compressed_produce():
    """Check if message is not gzipped if compression is disabled."""
    topic = "io-topic"
    kakfa_config = {"bootstrap.servers": "kafka:9092"}
    pub = MultiplexorPublisher(compression="gzip", **kakfa_config)

    pub.producer = MagicMock()
    pub.produce(json.dumps(INPUT_MSG).strip().encode(), topic)

    produce_mock_call = pub.producer.produce.call_args[0]
    assert topic == produce_mock_call[0]

    sent_data = json.loads(gzip.decompress(produce_mock_call[1]))
    assert sent_data == INPUT_MSG


def test_with_known_marks():
    """Check if the message is sent to the expected topics."""
    input_msg_bytes = json.dumps(INPUT_MSG).strip().encode()

    topics_mapping = {
        "MARK1": "topic1",
        "MARK2": "topic2",
    }

    kakfa_config = {"bootstrap.servers": "kafka:9092"}
    pub = MultiplexorPublisher(outgoing_topics=topics_mapping, **kakfa_config)

    pub.producer = MagicMock()

    pub.publish(INPUT_MSG, {"MARK1", "MARK2"})
    pub.producer.produce.assert_has_calls(
        [
            call("topic1", input_msg_bytes),
            call("topic2", input_msg_bytes),
        ],
        any_order=True,
    )


def test_with_unknown_marks():
    """Check if the message is not sent to any topics."""
    topics_mapping = {
        "MARK1": "topic1",
        "MARK2": "topic2",
    }

    kakfa_config = {"bootstrap.servers": "kafka:9092"}
    pub = MultiplexorPublisher(outgoing_topics=topics_mapping, **kakfa_config)

    pub.producer = MagicMock()

    pub.publish(INPUT_MSG, {"MARK3"})
    pub.producer.produce.assert_not_called()

    pub.publish(INPUT_MSG, set())
    pub.producer.produce.assert_not_called()



"""Contain unit tests for the ingress module."""

import pytest

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.ingress import parse_identity, parse_ingress_message


VALID_ENCODED_IDENTITIES = [
    "eyJpZGVudGl0eSI6IHsiaW50ZXJuYWwiOiB7Im9yZ19pZCI6ICIxMjM0NTY3OCJ9fX0K",
]


@pytest.mark.parametrize("valid_encoded_identity", VALID_ENCODED_IDENTITIES)
def test_parse_indentity_valid_values(valid_encoded_identity):
    """Check the behaviour with valid parameters of parse_identity."""
    identity = parse_identity(valid_encoded_identity)
    assert "identity" in identity
    assert "internal" in identity["identity"]
    assert "org_id" in identity["identity"]["internal"]


INVALID_IDENTITY_VALUES = [
    None,
    "asdasd",
    b"{}",
    b"SGVsbG8gd29ybGQ=",
    b"e30=",
]


@pytest.mark.parametrize("invalid_value", INVALID_IDENTITY_VALUES)
def test_parse_indentity_invalid_values(invalid_value):
    """Check that parse_identity catches all the exceptions and raises the expected one."""
    with pytest.raises(CCXMessagingError):
        parse_identity(invalid_value)


VALID_VALUES = [
    '{"url": "","b64_identity": "eyJpZGVudGl0eSI6IHsiaW50ZXJuYWwiOiB7Im9yZ19pZCI6ICIxMjM0NTY3OCJ9fX0K",'  # noqa E501
    '"timestamp": "2020-01-23T16:15:59.478901889Z"}',
    '{"url": "https://s3.com/hash", "unused-property": null, '
    '"b64_identity": "eyJpZGVudGl0eSI6IHsiaW50ZXJuYWwiOiB7Im9yZ19pZCI6ICIxMjM0NTY3OCJ9fX0K",'
    '"timestamp": "2020-01-23T16:15:59.478901889Z"}',
    '{"account":12345678, "url":"any/url", '
    '"b64_identity": "eyJpZGVudGl0eSI6IHsiaW50ZXJuYWwiOiB7Im9yZ19pZCI6ICIxMjM0NTY3OCJ9fX0K",'
    '"timestamp": "2020-01-23T16:15:59.478901889Z"}',
]


@pytest.mark.parametrize("valid_value", VALID_VALUES)
def test_parse_ingress_message_valid(valid_value):
    """Check when parse_identity works with valid values."""
    parsed_msg = parse_ingress_message(valid_value)

    assert "url" in parsed_msg
    assert "identity" in parsed_msg
    assert "timestamp" in parsed_msg


INVALID_VALUES = [
    None,
    1,
    10.0,
    "",
    "{}",
    '{"url":"any/url"}',
]


@pytest.mark.parametrize("invalid_value", INVALID_VALUES)
def test_parse_ingress_message_invalid(invalid_value):
    """Check when parse_identity works with valid values."""
    with pytest.raises(CCXMessagingError):
        parse_ingress_message(invalid_value)



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module implementing a ICM publisher to several Kafka topics depending on a filter."""

import gzip
import json
import logging
from typing import Any

from confluent_kafka import KafkaException, Producer
from insights_messaging.publishers import Publisher


log = logging.getLogger(__name__)
BEST_COMPRESSION = 9


class MultiplexorPublisher(Publisher):
    """A Kafka based publisher that sends the input message to the configured topics."""

    def __init__(
        self,
        outgoing_topics: dict[str, str] | None = None,
        kafka_broker_config: dict[str, str] | None = None,
        **kwargs,
    ) -> None:
        """Initialize a MultiplexorPublisher with the mapping between archive marks and topics.

        `outgoing_topics` is a dictionary that maps an archive "mark" (see `MultiplexorEngine`)
        to a Kafka topic. One archive can have several marks, so the input message can be sent
        to several Kafka topics.
        """
        self.topics_mapping = outgoing_topics if outgoing_topics is not None else {}

        if "compression" in kwargs:
            self.compression = kwargs.pop("compression")
        else:
            self.compression = None

        if kafka_broker_config:
            kwargs.update(kafka_broker_config)

        if "bootstrap.servers" not in kwargs:
            raise KafkaException("Broker not configured")

        log.debug(
            "Confluent Kafka consumer configuration arguments: Server: %s. Security protocol: %s.",
            kwargs.get("bootstrap.servers"),
            kwargs.get("security.protocol"),
        )
        self.producer = Producer(kwargs)
        log.info("Producing on brokers %s", kwargs.get("bootstrap.servers"))
        log.info("Topics mapping: %s", self.topics_mapping)

    def produce(self, outgoing_message: bytes, topic: str) -> None:
        """Send the message though the Kafka producer."""
        if self.compression:
            self.producer.produce(
                topic, gzip.compress(outgoing_message, compresslevel=BEST_COMPRESSION)
            )
        else:
            self.producer.produce(topic, outgoing_message)
        self.producer.poll(0)

    def publish(self, input_msg: dict[str, Any], response: set[str]) -> None:
        """Check to which topic should the `input_msg` be sent."""
        for mark in response:
            topic = self.topics_mapping.get(mark)
            if not topic:
                continue

            self.produce(json.dumps(input_msg).strip().encode(), topic)



# Copyright 2019, 2020, 2021, 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module that implements a custom Kafka publisher."""

import datetime
import json
import logging
from json import JSONDecodeError

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.publishers.kafka_publisher import KafkaPublisher


log = logging.getLogger(__name__)

RFC3339_FORMAT = "%Y-%m-%dT%H:%M:%SZ"


class RuleProcessingPublisher(KafkaPublisher):
    """RuleProcessingPublisher handles the results of the applied rules and publish them to Kafka.

    The results of the data analysis are received as a JSON (string)
    and turned into a byte array using UTF-8 encoding.
    The bytes are then sent to the output Kafka topic.

    Custom error handling for the whole pipeline is implemented here.
    """

    def __init__(self, outgoing_topic, kafka_broker_config=None, **kwargs):
        """Construct a new `RuleProcessingPublisher` given `kwargs` from the config YAML."""
        super().__init__(outgoing_topic, kafka_broker_config, **kwargs)

    def validate_timestamp_rfc3339(self, timestamp):
        """Check if the timestamp matches RFC3339 format."""
        try:
            datetime.datetime.strptime(timestamp, RFC3339_FORMAT)
        except:  # noqa E722
            return False
        return True

    def get_gathering_time(self, input_msg):
        """Retrieve the gathering time from input message if present, otherwise create one."""
        gathered_at = (
            input_msg.get("metadata", {}).get("custom_metadata", {}).get("gathering_time", None)
        )

        if not gathered_at:
            log.debug("Gathering time is not present; creating replacement")
            gathered_at = datetime.datetime.now().strftime(RFC3339_FORMAT)

        # If the timestamp is not in correct format, try to parse
        # format used in molodec. Otherwise use current timestamp.
        if not self.validate_timestamp_rfc3339(gathered_at):
            try:
                gathered_at = datetime.datetime.fromisoformat(gathered_at).strftime(RFC3339_FORMAT)
                log.debug("Converting gathering time from ISO format to RFC3339 format")
            except ValueError:
                log.debug("Gathering time could not be parsed; creating replacement")
                gathered_at = datetime.datetime.now().strftime(RFC3339_FORMAT)

        return gathered_at

    def publish(self, input_msg, report):
        """Publish an EOL-terminated JSON message to the output Kafka topic.

        The report is assumed to be a string representing a valid JSON object.
        A newline character will be appended to it, it will be converted into
        a byte array using UTF-8 encoding and the result of that will be sent
        to the producer to produce a message in the output Kafka topic.
        """
        try:
            report = json.loads(report)
        except (TypeError, json.decoder.JSONDecodeError):
            raise CCXMessagingError("Could not parse report; report is not in JSON format")

        if "reports" not in report.keys():
            log.debug("Report does not contain OCP rules related results; skipping")
            return

        report.pop("workload_recommendations", None)

        output_msg = {}
        try:
            org_id = int(input_msg["identity"]["identity"]["internal"]["org_id"])
        except (ValueError, KeyError, TypeError) as err:
            log.warning("Error extracting the OrgID: %s", err)
            raise CCXMessagingError("Error extracting the OrgID") from err

        try:
            account_number = int(input_msg["identity"]["identity"]["account_number"])
        except (ValueError, KeyError, TypeError) as err:
            log.warning("Error extracting the Account number: %s", err)
            account_number = None

        try:
            msg_timestamp = input_msg["timestamp"]
            msg_version = report.pop("version", 0)
            output_msg = {
                "OrgID": org_id,
                "ClusterName": input_msg["cluster_name"],
                "Report": report,
                "LastChecked": msg_timestamp,
                "Version": msg_version,
                "RequestId": input_msg.get("request_id"),
                "Metadata": {"gathering_time": self.get_gathering_time(input_msg)},
            }

            if account_number is not None:
                output_msg["AccountNumber"] = account_number

            message = json.dumps(output_msg) + "\n"

            log.debug("Sending response to the %s topic.", self.topic)
            # Convert message string into a byte array.
            self.produce(message.encode("utf-8"))
            log.debug("Message has been sent successfully.")
            log.debug(
                "Message context: OrgId=%s, AccountNumber=%s, "
                'ClusterName="%s", LastChecked="%s, Version=%d"',
                output_msg["OrgID"],
                output_msg.get("AccountNumber", ""),
                output_msg["ClusterName"],
                output_msg["LastChecked"],
                output_msg["Version"],
            )

        except KeyError as err:
            raise CCXMessagingError("Missing expected keys in the input message") from err

        except (TypeError, UnicodeEncodeError, JSONDecodeError) as err:
            log.warning(err)
            raise CCXMessagingError("Error encoding the response to publish") from err



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may naot use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module implementing a IDP publisher to Kafka topic."""

import json
from typing import Dict
import logging
from ccx_messaging.publishers.kafka_publisher import KafkaPublisher

LOG = logging.getLogger(__name__)


class SyncedArchivePublisher(KafkaPublisher):
    """Publisher for interanal data pipeline."""  # noqa: D203

    def publish(self, input_msg: Dict, report: str) -> None:
        """Publish response as Kafka message to outgoing topic."""
        output_msg = json.loads(report)
        output_msg.pop("reports", None)
        output_msg["request_id"] = input_msg.get("request_id")
        message = json.dumps(output_msg)

        LOG.debug("Sending response to the %s topic.", self.topic)
        # Convert message string into a byte array.
        self.produce(message.encode("utf-8"))
        LOG.debug("Message has been sent successfully.")
        LOG.debug(
            'Message context: path=%s, original_path=%s, metadata="%s"',
            output_msg["path"],
            output_msg["original_path"],
            output_msg["metadata"],
        )



# Copyright 2024 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""S3 Downloader for internal data pipeline."""

from contextlib import contextmanager

from insights_messaging.downloaders.s3 import S3Downloader as ICMS3Downloader

from ccx_messaging.error import CCXMessagingError


class S3Downloader(ICMS3Downloader):
    """Downloader for S3 bucket."""

    def __init__(self, **kwargs):
        """Set up the S3 downloader."""
        self.access_key = kwargs.get("access_key")
        self.secret_key = kwargs.get("secret_key")
        # 'endpoint_url' key is legacy and retained for backward compatibility.
        self.endpoint = kwargs.get("endpoint", kwargs.get("endpoint_url"))
        self.bucket = kwargs.get("bucket")
        if not self.access_key:
            raise CCXMessagingError("Access Key environment variable not set.")
        if not self.secret_key:
            raise CCXMessagingError("Secret Key environment variable not set.")
        if not self.endpoint:
            raise CCXMessagingError("Endpoint environment variable not set.")
        if not self.bucket:
            raise CCXMessagingError("Bucket environment variable not set.")
        super().__init__(
            key=self.access_key,
            secret=self.secret_key,
            client_kwargs={"endpoint_url": self.endpoint},
        )

    @contextmanager
    def get(self, path):
        """Download the archive from given path."""
        with super().get(f"{self.bucket}/{path}") as f:
            yield f



# Copyright 2025 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module that implements a custom Kafka publisher."""

import json
import logging
from typing import Any

import jsonschema

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.publishers.kafka_publisher import KafkaPublisher
from ccx_messaging.schemas import ARCHIVE_SYNCED_SCHEMA


log = logging.getLogger(__name__)


class IDPRuleProcessingPublisher(KafkaPublisher):
    """RuleProcessingPublisher handles the results of the applied rules and publish them to Kafka.

    The results of the data analysis are received as a JSON (string)
    and turned into a byte array using UTF-8 encoding.
    The bytes are then sent to the output Kafka topic.

    Custom error handling for the whole pipeline is implemented here.
    """

    def publish(self, input_msg: dict[str, Any], report: str | bytes) -> None:
        """Publish an EOL-terminated JSON message to the output Kafka topic.

        The report is assumed to be a string representing a valid JSON object.
        A newline character will be appended to it, it will be converted into
        a byte array using UTF-8 encoding and the result of that will be sent
        to the producer to produce a message in the output Kafka topic.
        """
        try:
            jsonschema.validate(input_msg, ARCHIVE_SYNCED_SCHEMA)

        except jsonschema.ValidationError as ex:
            raise CCXMessagingError("Invalid JSON format in the input message.") from ex

        try:
            report = json.loads(report)

        except (TypeError, json.decoder.JSONDecodeError):
            raise CCXMessagingError("Could not parse report; report is not in JSON format")

        output_msg = {
            "path": input_msg["path"],
            "metadata": input_msg["metadata"],
            "report": report,
            "request_id": input_msg.get("request_id"),
        }

        message = json.dumps(output_msg)
        log.debug("Sending response to the %s topic.", self.topic)
        # Convert message string into a byte array.
        self.produce(message.encode("utf-8"))
        log.debug("Message has been sent successfully.")



# Copyright 2024 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Class for s3 uploader."""

import logging
import boto3


LOG = logging.getLogger(__name__)


class S3Uploader:
    """S3 uploader."""

    def __init__(self, access_key, secret_key, endpoint, acl_enabled=True):
        """Inicialize uploader."""
        if not access_key:
            raise TypeError("access_key cannot be nulleable")

        if not secret_key:
            raise TypeError("secret_key cannot be nulleable")

        session = boto3.session.Session()

        self.client = session.client(
            service_name="s3",
            aws_access_key_id=access_key,
            aws_secret_access_key=secret_key,
            endpoint_url=endpoint,
        )

        self.acl_enabled = acl_enabled

    def upload_file(self, path, bucket, file_name):
        """Upload file to target path in bucket."""
        LOG.debug(f"Uploading '{file_name}' as '{path}' to '{bucket}'")

        with open(path, "rb") as file_data:
            if self.acl_enabled:
                self.client.put_object(
                    Bucket=bucket, Key=file_name, Body=file_data, ACL="bucket-owner-read"
                )
            else:
                self.client.put_object(Bucket=bucket, Key=file_name, Body=file_data)
            LOG.debug(f"Uploaded '{file_name}' as '{path}' to '{bucket}'")



# Copyright 2020 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module with utilities for the tests."""

import time


def mock_consumer_process_no_action_catch_exception(duration_s=0):
    """Mock the process method of ICM consumer so it does nothing.

    This mocks the process method and removes all processing, but
    still catches the raised exceptions.
    If duration_s is provided, the process method is made to sleep
    for the given amount of seconds before returning, unless something
    raises an exception within that timeframe.
    """
    try:
        time.sleep(duration_s)
    except Exception:
        raise



"""Module containing the consumer for the Kafka topic produced by the Rules Processing service."""

import json
import logging
import time
import os
from tempfile import NamedTemporaryFile
from typing import Any


from confluent_kafka import KafkaException
from insights.core.exceptions import InvalidContentType

from ccx_messaging.consumers.kafka_consumer import KafkaConsumer
from ccx_messaging.error import CCXMessagingError
from ccx_messaging.internal_pipeline import parse_rules_results_msg
from ccx_messaging.monitored_broker import SentryMonitoredBroker


LOG = logging.getLogger(__name__)


class RulesResultsConsumer(KafkaConsumer):
    """Consumer for the topic produced by `idp_rule_processing_publisher.IDPRuleProcessingPublisher`."""  # noqa: E501

    def get_url(self, input_msg: dict[str:str]) -> str:
        """Retrieve path to the archive in the S3 storage from dictionary."""
        return input_msg["file_path"]

    def process_msg(self, msg):
        """Process a single message received from the topic."""
        if not msg:
            LOG.debug("Empty record. Should not happen")
            return

        self.last_received_message_time = time.time()  # Base class thread control
        if msg.error():
            raise KafkaException(msg.error())

        try:
            # Deserialize
            value = self.deserialize(msg)
            with NamedTemporaryFile(mode="w", encoding="utf-8") as temp_file:
                json.dump(value, temp_file)
                temp_file.flush()
                # Extend original Kafka message with path to temporary file
                value["file_path"] = temp_file.name
                # Core Messaging process
                self.process(value)

        except InvalidContentType as ex:
            LOG.warning("The archive cannot be processed by Insights: %s", ex)
            self.process_dead_letter(msg)

        except CCXMessagingError as ex:
            LOG.warning(
                "Unexpected error processing incoming message. (%s): %s. Error: %s",
                self.log_pattern,
                msg.value(),
                ex,
            )
            self.process_dead_letter(msg)

        except TimeoutError as ex:
            self.fire("on_process_timeout")
            LOG.warning("Process timeout: %s", ex)
            self.process_dead_letter(msg)

        except Exception as ex:  # pylint: disable=broad-exception-caught
            LOG.exception(ex)
            self.process_dead_letter(msg)

    def deserialize(self, msg):
        """Deserialize the message received from Kafka into a dictionary."""
        if not msg:
            raise CCXMessagingError("No incoming message: %s", msg)

        try:
            value = msg.value()
        except AttributeError as ex:
            raise CCXMessagingError("Invalid incoming message type: %s", type(msg)) from ex

        LOG.debug("Deserializing incoming message(%s): %s", self.log_pattern, value)

        if not value:
            raise CCXMessagingError("Unable to read incoming message: %s", value)

        deseralized_msg = parse_rules_results_msg(value)
        LOG.debug("JSON message deserialized (%s): %s", self.log_pattern, deseralized_msg)
        return deseralized_msg

    def create_broker(self, input_msg: dict[str, Any]) -> SentryMonitoredBroker:
        """Create a suitable `Broker`."""
        broker = SentryMonitoredBroker()
        broker["cluster_id"] = input_msg["metadata"]["cluster_id"]
        broker["report_path"] = self.create_report_path(input_msg)
        return broker

    def create_report_path(self, input_msg):
        """Construct report path from the original archive path."""
        path, file_name = os.path.split(input_msg.get("path"))
        archive_name = file_name.replace(".tar.gz", "")
        new_path = path.replace("archives/compressed", "insights")
        report_path = os.path.join(new_path, archive_name, "insights.json")
        return report_path



"""A string template with slice substitutions."""

from __future__ import annotations

import re
import string
from collections import ChainMap
from dataclasses import dataclass


_sentinel_map = {}


@dataclass
class SliceSelector:
    """Representation of a slice substitution."""

    name: str
    initial: int
    final: int
    element: int

    @classmethod
    def from_match(cls, match_: re.Match) -> SliceSelector:
        """Create a SliceSelector from a pattern match."""
        element = match_.group("element")
        element = int(element) if element else None
        initial = match_.group("initial")
        initial = int(initial) if initial else None
        final = match_.group("final")
        final = int(final) if final else None
        var_id = match_.group("identifier")

        return SliceSelector(
            var_id,
            initial,
            final,
            element,
        )


class SlicedTemplate(string.Template):
    """A template class for support $-substitutions with element slicing."""

    idpattern = r"(?a:[_a-z][_a-z0-9]*((\[(-?\d+)?:(-?\d+)?\])|(\[-?\d+\]))?)"
    idpattern_with_slice = re.compile(
        r"(?P<identifier>[_a-z][_a-z0-9]*)"
        r"(?:(?:\[(?P<initial>-?\d+)?:(?P<final>-?\d+)?\])|"
        r"(?:\[(?P<element>-?\d+)\]))"
    )

    def __init__(self, template):
        """Initialise the SliceTemplate object."""
        super().__init__(template)
        self.slice_selectors = {}

        for identifier in super().get_identifiers():
            match_ = SlicedTemplate.idpattern_with_slice.match(identifier)
            if not match_:
                continue

            self.slice_selectors[identifier] = SliceSelector.from_match(match_)

    def substitute(self, mapping=_sentinel_map, /, **kws) -> str:
        """Perform the substitution using the mapping dictionary and named arguments.

        It fails when a template identifier is not found.
        """
        if mapping is _sentinel_map:
            mapping = kws

        elif kws:
            mapping = ChainMap(mapping, kws)

        for identifier, selector in self.slice_selectors.items():
            if selector.element:
                slice_value = mapping[selector.name][selector.element]
            else:
                slice_value = mapping[selector.name][selector.initial : selector.final]

            mapping[identifier] = slice_value

        return super().substitute(mapping)

    def safe_substitute(self, mapping=_sentinel_map, /, **kws) -> str:
        """Perform the substitution using the mapping dictionary and named arguments.

        It will ignore substitutions if the referenced identifier is not found.
        """
        if mapping is _sentinel_map:
            mapping = kws

        elif kws:
            mapping = ChainMap(mapping, kws)

        for identifier, selector in self.slice_selectors.items():
            if selector.name not in mapping:
                continue

            if selector.element:
                slice_value = mapping[selector.name][selector.element]
            else:
                slice_value = mapping[selector.name][selector.initial : selector.final]

            mapping[identifier] = slice_value

        return super().safe_substitute(mapping)

    def get_identifiers(self) -> list[str]:
        """Return a list of the valid identifiers in the template."""
        retval = set(super().get_identifiers()) - set(self.slice_selectors.keys())

        for selector in self.slice_selectors.values():
            retval.add(selector.name)

        return list(retval)



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module including an extension to the Insights Core Messaging ConsumerWatcher class."""

from insights_messaging.watchers import ConsumerWatcher as ICMConsumerWatcher


class ConsumerWatcher(ICMConsumerWatcher):
    """Extension of the ConsumerWatcher class to allow handling new callbacks."""

    def on_not_handled(self, input_msg):
        """Handle a fired event after a received element cannot be handled by the consumer."""



# Copyright 2024 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Unit tests for s3_upload_engine."""

import json
import os
import pytest
from unittest.mock import MagicMock

from ccx_messaging.engines.s3_upload_engine import S3UploadEngine
from ccx_messaging.utils.s3_uploader import S3Uploader


BROKER = {
    "cluster_id": "11111111-2222-3333-4444-555555555555",
    "org_id": "00000000",
    "original_path": "00000000/11111111-2222-3333-4444-555555555555/66666666666666-77777777777777777777777777777777",  # noqa: E501
    "year": "6666",
    "month": "66",
    "day": "66",
    "time": "666666",
    "hour": "66",
    "minute": "66",
    "second": "66",
    "id": "77777777777777777777777777777777",
}


BROKER2 = {
    "cluster_id": "22222222-3333-4444-5555-666666666666",
    "org_id": "00000000",
    "original_path": "00000000/22222222-3333-4444-5555-666666666666/77777777777777-88888888888888888888888888888888",  # noqa: E501
    "year": "7777",
    "month": "77",
    "day": "77",
    "time": "777777",
    "hour": "77",
    "minute": "77",
    "second": "77",
    "id": "88888888888888888888888888888888",
}


METADATA = {
    "path": "archives/compressed/11/11111111-2222-3333-4444-555555555555/666666/66/666666.tar.gz",
    "original_path": "00000000/11111111-2222-3333-4444-555555555555/66666666666666-77777777777777777777777777777777",  # noqa: E501
    "metadata": {
        "cluster_id": "11111111-2222-3333-4444-555555555555",
        "external_organization": "00000000",
    },
}


LOCAL_FILE_PATH = "file_path"
DEST_BUCKET = "bucket"


def test_init():
    """Test inicialization of engine."""
    engine = S3UploadEngine(
        None,
        access_key="test",
        secret_key="test",
        endpoint="https://s3.amazonaws.com",
    )
    assert engine is not None


def test_bad_key():
    """Test inicialization of engine with bad key."""
    with pytest.raises(TypeError):
        S3UploadEngine(
            None,
            access_key="",
            secret_key="",
            endpoint="https://s3.amazonaws.com",
        )


def test_bad_url():
    """Test inicialization of engine with bad URL."""
    with pytest.raises(ValueError):
        S3UploadEngine(
            None,
            access_key="test",
            secret_key="test",
            endpoint="BAD_URL",
        )


def test_engine_metadata():
    """Test of engines processing of metadata."""
    engine = S3UploadEngine(
        None,
        access_key="test",
        secret_key="test",
        endpoint="https://s3.amazonaws.com",
        dest_bucket=DEST_BUCKET,
        archives_path_prefix="archives/compressed",
    )
    engine.uploader = MagicMock()
    metadata = engine.process(BROKER, LOCAL_FILE_PATH)
    assert metadata == json.dumps(METADATA)


def test_engine_upload_file():
    """Test if upload_file metode was called with correct arguments."""
    engine = S3UploadEngine(
        None,
        access_key="test",
        secret_key="test",
        endpoint="https://s3.amazonaws.com",
        dest_bucket=DEST_BUCKET,
    )
    engine.uploader = MagicMock()
    S3Uploader.client = MagicMock()
    metadata = engine.process(BROKER2.copy(), LOCAL_FILE_PATH)
    metadata = json.loads(metadata)
    _, args, _ = engine.uploader.mock_calls[0]
    assert args[0] == LOCAL_FILE_PATH
    assert args[1] == DEST_BUCKET
    assert args[2] == metadata.get("path")


def test_uploader():
    """Test of uploader."""
    file = open(LOCAL_FILE_PATH, "a")
    file.write("test data")
    file.close()
    uploader = S3Uploader(access_key="test", secret_key="test", endpoint="https://s3.amazonaws.com")
    uploader.client.put_object = MagicMock()
    uploader.upload_file(path="file_path", bucket=DEST_BUCKET, file_name=METADATA.get("path"))
    os.remove(LOCAL_FILE_PATH)
    _, _, kwargs = uploader.client.put_object.mock_calls[0]
    assert kwargs.get("Bucket") == DEST_BUCKET
    assert kwargs.get("Key") == METADATA.get("path")


def test_uploader_no_existing_file():
    """Test uploading nonexistent file."""
    with pytest.raises(FileNotFoundError):
        uploader = S3Uploader(
            access_key="test", secret_key="test", endpoint="https://s3.amazonaws.com"
        )
        uploader.client.put_object = MagicMock()
        uploader.upload_file(path="file_path", bucket=DEST_BUCKET, file_name=METADATA.get("path"))


def test_path_using_timestamp():
    """Check path creation using timestamp elements."""
    engine = S3UploadEngine(
        None,
        access_key="test",
        secret_key="test",
        endpoint="https://s3.amazonaws.com",
        dest_bucket=DEST_BUCKET,
        archive_name_pattern="$year/$month/$day/$cluster_id-$id.tar.gz",
    )
    engine.uploader = MagicMock()
    S3Uploader.client = MagicMock()

    report = engine.process(BROKER2.copy(), LOCAL_FILE_PATH)
    report = json.loads(report)
    assert (
        report.get("path")
        == "7777/77/77/22222222-3333-4444-5555-666666666666-88888888888888888888888888888888.tar.gz"
    )



# Copyright 2022 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Helper class for testing string handling."""


class UnicodeEncodeErrorThrower:
    """UnicodeErrorThrower is helper class for testing string handling."""

    def __init__(self):
        """Initialize new constructed instance of UnicodeErrorThrower."""
        pass

    def encode(self, encoding):
        """Simulate method encode of type str."""
        raise UnicodeEncodeError(encoding, "can not encode", 0, 1, "can not encode")



# Copyright 2023 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for the DVOMetricsPublisher class."""

import json
from unittest.mock import MagicMock, patch

import freezegun
import pytest
from confluent_kafka import KafkaException

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.publishers.dvo_metrics_publisher import DVOMetricsPublisher


def test_init():
    """Check that init creates a valid object."""
    kakfa_config = {
        "bootstrap.servers": "kafka:9092",
    }
    DVOMetricsPublisher(outgoing_topic="topic name", **kakfa_config)


INVALID_TOPIC_NAMES = [
    None,
    b"Topic name",
    [],
    {},
    4,
    5.5,
    5 + 2j,
]


@pytest.mark.parametrize("topic_name", INVALID_TOPIC_NAMES)
def test_init_invalid_topic(topic_name):
    """Check what happens when the output_topic parameter is not valid."""
    with pytest.raises(CCXMessagingError):
        DVOMetricsPublisher(topic_name)


INVALID_KWARGS = [
    {},
    {"bootstrap_servers": "kafka:9092"},
    {"bootstrap.servers": "kafka:9092", "unknown_option": "value"},
]


@pytest.mark.parametrize("kwargs", INVALID_KWARGS)
def test_bad_initialization(kwargs):
    """Check that init fails when using not valid kwargs."""
    with pytest.raises(KafkaException):
        DVOMetricsPublisher(outgoing_topic="topic", **kwargs)


@pytest.mark.parametrize("kafka_broker_cfg", INVALID_KWARGS)
@pytest.mark.parametrize("kwargs", INVALID_KWARGS)
def test_bad_init_with_kafka_config(kafka_broker_cfg, kwargs):
    """Check that init fails when using not valid kwargs."""
    with pytest.raises(KafkaException):
        DVOMetricsPublisher(outgoing_topic="topic", **kwargs)


INVALID_INPUT_MSGS = [
    None,
    "",
    1,
    2.0,
    3 + 1j,
    [],
    {},  # right type, missing identity
    {"identity": {}},  # missing identity-identity
    {"identity": {"identity": {}}},  # missing identity-identity-internal
    {"identity": {"identity": {"internal": {}}}},  # missing identity-identity-internal-org_id
    {"identity": {"identity": {"internal": {"org_id": 15.2}}}},  # incorrect org_id type
    {"identity": {"identity": {"internal": {"org_id": 10}}}},  # missing timestamp
    {
        "identity": {
            "identity": {
                "internal": {"org_id": 10},
                "account_number": 1 + 2j,  # missing timestamp
            },
        },
    },
    {
        "identity": {
            "identity": {
                "internal": {"org_id": 10},
                "account_number": 1,
            },
        },
    },  # missing timestamp
    {
        "identity": {
            "identity": {
                "internal": {"org_id": 10},
                "account_number": 1,
            },
        },
        "timestamp": "a timestamp",
    },  # missing cluster_name
]


@pytest.mark.parametrize("wrong_input_msg", INVALID_INPUT_MSGS)
def test_publish_bad_argument(wrong_input_msg):
    """Check that invalid messages passed by the framework are handled gracefully."""
    sut = DVOMetricsPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})
    sut.producer = MagicMock()

    with pytest.raises(CCXMessagingError):
        sut.publish(wrong_input_msg, {})
        assert not sut.producer.produce.called


VALID_INPUT_MSG = [
    pytest.param(
        {
            "identity": {
                "identity": {
                    "internal": {"org_id": 10},
                    "account_number": 1,
                },
            },
            "timestamp": "a timestamp",
            "cluster_name": "uuid",
            "request_id": "a request id",
            "topic": "incoming_topic",
            "partition": 0,
            "offset": 100,
        },
        {
            "OrgID": 10,
            "ClusterName": "uuid",
            "Metrics": {"workload_recommendations": []},
            "RequestId": "a request id",
            "LastChecked": "a timestamp",
            "Version": 0,
            "AccountNumber": 1,
        },
        id="with account",
    ),
    pytest.param(
        {
            "identity": {
                "identity": {
                    "internal": {"org_id": 10},
                    "account_number": 5 + 2j,
                },
            },
            "timestamp": "a timestamp",
            "cluster_name": "uuid",
            "request_id": "a request id",
            "topic": "incoming_topic",
            "partition": 0,
            "offset": 100,
        },
        {
            "OrgID": 10,
            "ClusterName": "uuid",
            "Metrics": {"workload_recommendations": []},
            "RequestId": "a request id",
            "LastChecked": "a timestamp",
            "Version": 0,
        },
        id="invalid account",
    ),
    pytest.param(
        {
            "identity": {
                "identity": {
                    "internal": {"org_id": 10},
                    "account_number": "",
                },
            },
            "timestamp": "a timestamp",
            "cluster_name": "uuid",
            "request_id": "a request id",
            "topic": "incoming_topic",
            "partition": 0,
            "offset": 100,
        },
        {
            "OrgID": 10,
            "ClusterName": "uuid",
            "Metrics": {"workload_recommendations": []},
            "RequestId": "a request id",
            "LastChecked": "a timestamp",
            "Version": 0,
        },
        id="empty account",
    ),
    pytest.param(
        {
            "identity": {
                "identity": {
                    "internal": {"org_id": 10},
                },
            },
            "timestamp": "a timestamp",
            "cluster_name": "uuid",
            "request_id": "a request id",
            "topic": "incoming_topic",
            "partition": 0,
            "offset": 100,
        },
        {
            "OrgID": 10,
            "ClusterName": "uuid",
            "Metrics": {"workload_recommendations": []},
            "RequestId": "a request id",
            "LastChecked": "a timestamp",
            "Version": 0,
        },
        id="no account",
    ),
    pytest.param(
        {
            "identity": {
                "identity": {
                    "internal": {"org_id": 10},
                },
            },
            "timestamp": "a timestamp",
            "cluster_name": "uuid",
            "request_id": "a request id",
            "topic": "incoming_topic",
            "partition": 0,
            "offset": 100,
            "metadata": {"custom_metadata": {"gathering_time": "2023-08-14T09:31:46Z"}},
        },
        {
            "OrgID": 10,
            "ClusterName": "uuid",
            "Metrics": {"workload_recommendations": []},
            "RequestId": "a request id",
            "LastChecked": "a timestamp",
            "Version": 0,
        },
        id="with gathering timestamp",
    ),
    pytest.param(
        {
            "identity": {
                "identity": {
                    "internal": {"org_id": 10},
                },
            },
            "timestamp": "a timestamp",
            "cluster_name": "uuid",
            "request_id": "a request id",
            "topic": "incoming_topic",
            "partition": 0,
            "offset": 100,
            "metadata": {"custom_metadata": {"gathering_time": "2023-08-14T09:31:46.677052"}},
        },
        {
            "OrgID": 10,
            "ClusterName": "uuid",
            "Metrics": {"workload_recommendations": []},
            "RequestId": "a request id",
            "LastChecked": "a timestamp",
            "Version": 0,
        },
        id="with gathering timestamp in ISO format",
    ),
    pytest.param(
        {
            "identity": {
                "identity": {
                    "internal": {"org_id": 10},
                },
            },
            "timestamp": "a timestamp",
            "cluster_name": "uuid",
            "request_id": "a request id",
            "topic": "incoming_topic",
            "partition": 0,
            "offset": 100,
            "metadata": {"custom_metadata": {"some_timestamp": "2023-08-14T09:31:46.677052"}},
        },
        {
            "OrgID": 10,
            "ClusterName": "uuid",
            "Metrics": {"workload_recommendations": []},
            "RequestId": "a request id",
            "LastChecked": "a timestamp",
            "Version": 0,
        },
        id="with custom metadata without gathering timestamp",
    ),
    pytest.param(
        {
            "identity": {
                "identity": {
                    "internal": {"org_id": 10},
                },
            },
            "timestamp": "a timestamp",
            "cluster_name": "uuid",
            "request_id": "a request id",
            "topic": "incoming_topic",
            "partition": 0,
            "offset": 100,
            "metadata": {},
        },
        {
            "OrgID": 10,
            "ClusterName": "uuid",
            "Metrics": {"workload_recommendations": []},
            "RequestId": "a request id",
            "LastChecked": "a timestamp",
            "Version": 0,
        },
        id="empty metadata",
    ),
]


@freezegun.freeze_time("2012-01-14")
@pytest.mark.parametrize("input, expected_output", VALID_INPUT_MSG)
def test_publish_valid(input, expected_output):
    """Check that Kafka producer is called with an expected message."""
    report = '{"workload_recommendations": []}'

    expected_output = json.dumps(expected_output) + "\n"
    sut = DVOMetricsPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})
    sut.producer = MagicMock()

    sut.publish(input, report)
    sut.producer.produce.assert_called_with("outgoing_topic", expected_output.encode())


INVALID_REPORTS = [
    None,
    1,
    2.0,
    1 + 3j,
    [],
    {},
    "",
]


@pytest.mark.parametrize("invalid_report", INVALID_REPORTS)
def test_publish_invalid_report(invalid_report):
    """Check the behaviour of publish when an invalid report is received."""
    sut = DVOMetricsPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})
    sut.producer = MagicMock()

    with pytest.raises(CCXMessagingError):
        sut.publish(VALID_INPUT_MSG, invalid_report)
        assert not sut.producer.produce.called


@freezegun.freeze_time("2012-01-14")
@pytest.mark.parametrize("input,output", VALID_INPUT_MSG)
def test_error(input, output):
    """Check that error just prints a log."""
    _ = output  # output values are not needed

    sut = DVOMetricsPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})

    with patch("ccx_messaging.publishers.kafka_publisher.log") as log_mock:
        sut.error(input, None)
        assert log_mock.warning.called


VALID_REPORTS = [
    pytest.param(
        json.dumps(
            {"version": 1, "reports": [], "pass": [], "info": [], "workload_recommendations": []}
        ),
        {
            "OrgID": 10,
            "ClusterName": "uuid",
            "Metrics": {"pass": [], "info": [], "workload_recommendations": []},
            "RequestId": "a request id",
            "LastChecked": "a timestamp",
            "Version": 1,
            "AccountNumber": 1,
        },
        id="valid_report",
    )
]


@pytest.mark.parametrize("input,expected_output", VALID_REPORTS)
def test_filter_dvo_results(input, expected_output):
    """Check that the external rule reports are filtered out from the engine results."""
    sut = DVOMetricsPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})
    sut.producer = MagicMock()
    expected_output = json.dumps(expected_output) + "\n"

    sut.publish(VALID_INPUT_MSG[0][0][0], input)
    sut.producer.produce.assert_called_with("outgoing_topic", expected_output.encode())


def test_empty_dvo_results():
    """Check that the publisher does not send empty message."""
    sut = DVOMetricsPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})
    sut.producer = MagicMock()

    input = json.dumps({"version": 1, "reports": [], "pass": [], "info": []})
    sut.publish(VALID_INPUT_MSG[0][0][0], input)
    assert not sut.producer.produce.called



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module including instrumentation to expose retrieved stats to Prometheus."""

import logging
import os
import tarfile
import time
import json

from insights.core.archives import TarExtractor, ZipExtractor
from insights_messaging.watchers import EngineWatcher
from prometheus_client import Counter, Histogram, start_http_server, REGISTRY

from ccx_messaging.watchers.consumer_watcher import ConsumerWatcher


LOG = logging.getLogger(__name__)
# Label to differentiate between OCP, OLS and HyperShift tarballs
ARCHIVE_TYPE_LABEL = "archive"
ARCHIVE_TYPE_VALUES = ["ocp", "hypershift", "ols"]
IO_GATHERING_REMOTE_CONFIG_LABEL = "version"


# pylint: disable=too-many-instance-attributes
class StatsWatcher(ConsumerWatcher, EngineWatcher):
    """A Watcher that stores different Prometheus `Counter`s."""

    def __init__(self, prometheus_port=8000):
        """Create the needed Counter objects and start serving Prometheus stats."""
        super().__init__()

        self._recv_total = Counter(
            "ccx_consumer_received_total", "Counter of received Kafka messages"
        )

        self._filtered_total = Counter(
            "ccx_consumer_filtered_total", "Counter of filtered Kafka messages"
        )

        self._downloaded_total = Histogram(
            "ccx_downloaded_total", "Histogram of the size of downloaded items"
        )

        self._archive_size = Histogram(
            "ccx_consumer_archive_size",
            "Histogram of the size of downloaded archive with archive type label",
            [ARCHIVE_TYPE_LABEL],
        )

        self._extracted_total = Counter(
            "ccx_consumer_extracted_total",
            "Counter of extracted archives",
            [ARCHIVE_TYPE_LABEL],
        )

        self._processed_total = Counter(
            "ccx_engine_processed_total",
            "Counter of files processed by the OCP Engine",
            [ARCHIVE_TYPE_LABEL],
        )

        self._published_total = Counter(
            "ccx_published_total",
            "Counter of reports successfully published",
            [ARCHIVE_TYPE_LABEL],
        )

        self._failures_total = Counter(
            "ccx_failures_total",
            "Counter of failures during the pipeline",
            [ARCHIVE_TYPE_LABEL],
        )

        self._not_handling_total = Counter(
            "ccx_not_handled_total",
            "Counter of received elements that are not handled by the pipeline",
        )

        self._download_duration = Histogram(
            "ccx_download_duration_seconds", "Histogram of archive download durations"
        )

        self._process_duration = Histogram(
            "ccx_process_duration_seconds",
            "Histogram of durations of processing archives by the OCP engine",
            [ARCHIVE_TYPE_LABEL],
        )

        self._publish_duration = Histogram(
            "ccx_publish_duration_seconds",
            "Histogram of durations of publishing the OCP engine results",
            [ARCHIVE_TYPE_LABEL],
        )

        self._processed_timeout_total = Counter(
            "ccx_engine_processed_timeout_total",
            "Counter of timeouts while processing archives",
            [ARCHIVE_TYPE_LABEL],
        )

        self._gathering_conditions_remote_configuration_version = Counter(
            "gathering_conditions_remote_configuration_version",
            "Counter of times a given configuration is seen",
            [IO_GATHERING_REMOTE_CONFIG_LABEL],
        )

        self._start_time = None
        self._downloaded_time = None
        self._processed_time = None
        self._published_time = None

        # Archive type used in the metrics is set within on_extract, as we need
        # to extract the archive in order to know that information
        # TODO: Change to archive metadata dict, with archive type and archive size
        self._archive_metadata = {"type": "ocp", "size": 0, "s3_path": ""}

        self._initialize_metrics_with_labels()

        start_http_server(prometheus_port)
        LOG.info("StatWatcher created and listening on port %s", prometheus_port)

    def on_recv(self, input_msg):
        """On received event handler."""
        LOG.debug("Receiving 'on_recv' callback")
        self._recv_total.inc()

        self._start_time = time.time()
        self._reset_times()
        self._reset_archive_metadata()

        if "path" in input_msg:
            self._archive_metadata["s3_path"] = input_msg["path"]
        elif "url" in input_msg:
            # this means we are consuming directly from ingress (external
            # pipeline), so printing the URL would include some short-live
            # credentials used to access the archive. It's a risk to log this
            pass
        else:
            LOG.error(f"message has no S3 path: {input_msg}")

    def on_filter(self):
        """On filter event handler."""
        LOG.debug("Receiving 'on_filter' callback")
        self._filtered_total.inc()

    def on_extract(
        self, ctx, broker, extraction: tarfile.TarFile | TarExtractor | ZipExtractor
    ) -> None:
        """On extract event handler for extractor using engines."""
        LOG.debug("Receiving 'on_extract' callback")

        if isinstance(extraction, tarfile.TarFile):
            self.on_extract_with_tarfile(extraction)

        else:
            self.on_extract_with_extractor(extraction)

    def on_extract_with_tarfile(self, extraction: tarfile.TarFile) -> None:
        """On extract event handler for engines using tarfile."""
        tarfile_contents = extraction.getnames()

        if "openshift_lightspeed.json" in tarfile_contents:
            self._archive_metadata["type"] = "ols"
        elif os.path.join("config", "infrastructure.json") in tarfile_contents:
            self._archive_metadata["type"] = "hypershift"

    def on_extract_with_extractor(self, extraction: TarExtractor | ZipExtractor) -> None:
        """On extract event handler for engines using extractor."""
        # Set archive_type label based on found file
        if os.path.exists(os.path.join(extraction.tmp_dir, "openshift_lightspeed.json")):
            self._archive_metadata["type"] = "ols"
        elif os.path.exists(os.path.join(extraction.tmp_dir, "config", "infrastructure.json")):
            self._archive_metadata["type"] = "hypershift"

        self._extracted_total.labels(**{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}).inc()

        self._archive_size.labels(**{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}).observe(
            self._archive_metadata["size"]
        )

        # Set the IO remote configuration version
        remote_config_path = os.path.join(
            extraction.tmp_dir, "insights-operator", "remote-configuration.json"
        )
        try:
            with open(remote_config_path, "rb") as f:
                data = json.load(f)
                version = data["version"]
                self._gathering_conditions_remote_configuration_version.labels(version).inc()
        except FileNotFoundError:
            LOG.debug("this archive didn't use remote-configurations")
        except KeyError:
            LOG.warning(
                f"archive {self._archive_metadata['s3_path']} doesn't have "
                "a version in the remote-configuration"
            )
            self._gathering_conditions_remote_configuration_version.labels("None").inc()
        except Exception as e:
            LOG.error("cannot read remote-configuration.json", exc_info=e)

    def on_download(self, path):
        """On downloaded event handler."""
        LOG.debug("Receiving 'on_download' callback")
        archive_size = os.path.getsize(path)

        self._archive_metadata["size"] = archive_size
        self._downloaded_total.observe(archive_size)

        self._downloaded_time = time.time()
        self._download_duration.observe(self._downloaded_time - self._start_time)

    def on_process(self, input_msg, results):
        """On processed event handler."""
        LOG.debug("Receiving 'on_process' callback")
        self._processed_total.labels(**{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}).inc()

        self._processed_time = time.time()
        self._process_duration.labels(
            **{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}
        ).observe(self._processed_time - self._downloaded_time)

    def on_process_timeout(self):
        """On process timeout event handler."""
        LOG.debug("Receiving 'on_process_timeout' callback")
        self._processed_timeout_total.labels(
            **{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}
        ).inc()

    def on_consumer_success(self, input_msg, broker, results):
        """On consumer success event handler."""
        LOG.debug("Receiving 'on_consumer_success' callback")
        self._published_total.labels(**{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}).inc()

        self._published_time = time.time()
        self._publish_duration.labels(
            **{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}
        ).observe(self._published_time - self._processed_time)

    def on_consumer_failure(self, input_msg, exception):
        """On consumer failure event handler."""
        LOG.debug("Receiving 'on_consumer_failure' callback")
        self._failures_total.labels(**{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}).inc()

        if self._downloaded_time is None:
            self._download_duration.observe(time.time() - self._start_time)
            self._process_duration.labels(
                **{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}
            ).observe(0)
        elif self._processed_time is None:
            self._process_duration.labels(
                **{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}
            ).observe(time.time() - self._downloaded_time)

        self._publish_duration.labels(
            **{ARCHIVE_TYPE_LABEL: self._archive_metadata["type"]}
        ).observe(0)

    def on_not_handled(self, input_msg):
        """On not handled messages success event handler."""
        LOG.debug("Receiving 'on_not_handled' callback")
        self._not_handling_total.inc()

    def _reset_times(self):
        """Set all timestamps with the exception of start time to `None`."""
        self._downloaded_time = None
        self._processed_time = None
        self._published_time = None

    def _reset_archive_metadata(self):
        """Reset the cached archive metadata."""
        self._archive_metadata["type"] = "ocp"
        self._archive_metadata["size"] = 0
        self._archive_metadata["s3_path"] = ""

    def _initialize_metrics_with_labels(self):
        """Initialize Prometheus metrics that have at least one label.

        Metrics with labels are not initialized when declared, because the Prometheus
        client can’t know what values the label can have. This is therefore needed in
        order to initialize them.
        """
        for val in ARCHIVE_TYPE_VALUES:
            self._extracted_total.labels(val)
            self._archive_size.labels(val)
            self._processed_total.labels(val)
            self._published_total.labels(val)
            self._failures_total.labels(val)
            self._process_duration.labels(val)
            self._publish_duration.labels(val)
            self._processed_timeout_total.labels(val)

    def __del__(self):
        """Destructor for handling counters unregistering."""
        REGISTRY.unregister(self._recv_total)
        REGISTRY.unregister(self._filtered_total)
        REGISTRY.unregister(self._downloaded_total)
        REGISTRY.unregister(self._archive_size)
        REGISTRY.unregister(self._extracted_total)
        REGISTRY.unregister(self._processed_total)
        REGISTRY.unregister(self._published_total)
        REGISTRY.unregister(self._failures_total)
        REGISTRY.unregister(self._not_handling_total)
        REGISTRY.unregister(self._download_duration)
        REGISTRY.unregister(self._process_duration)
        REGISTRY.unregister(self._publish_duration)
        REGISTRY.unregister(self._processed_timeout_total)
        REGISTRY.unregister(self._gathering_conditions_remote_configuration_version)



# Copyright 2023 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module that implements a customizable Kafka publisher."""

import logging
import gzip

from confluent_kafka import KafkaException, Producer
from insights_messaging.publishers import Publisher

from ccx_messaging.error import CCXMessagingError

log = logging.getLogger(__name__)
BEST_COMPRESSION = 9


class KafkaPublisher(Publisher):
    """KafkaPublisher is a base class for Kafka based publishers.

    It relays on Confluent Kafka library to perform the Kafka related
    tasks.

    This is an "abstract" class, so it needs to be extended with the
    proper methods.
    """

    def __init__(self, outgoing_topic: str, kafka_broker_config: dict = None, **kwargs):
        """Construct a new `KafkaPubisher` given `kwargs` from the config YAML."""
        if not isinstance(outgoing_topic, str):
            raise CCXMessagingError("outgoing_topic should be a str")

        self.topic = outgoing_topic
        if "compression" in kwargs:
            self.compression = kwargs.pop("compression")
        else:
            self.compression = None

        if kafka_broker_config:
            kwargs.update(kafka_broker_config)

        if "bootstrap.servers" not in kwargs:
            raise KafkaException("Broker not configured")

        log.debug(
            "Confluent Kafka consumer configuration arguments: "
            "Server: %s. "
            "Topic: %s. "
            "Security protocol: %s.",
            kwargs.get("bootstrap.servers"),
            self.topic,
            kwargs.get("security.protocol"),
        )
        self.producer = Producer(kwargs)
        log.info(
            "Producing to topic '%s' on brokers %s", self.topic, kwargs.get("bootstrap.servers")
        )

    def produce(self, outgoing_message: bytes):
        """Send the message though the Kafka producer."""
        if self.compression:
            self.producer.produce(
                self.topic, gzip.compress(outgoing_message, compresslevel=BEST_COMPRESSION)
            )
        else:
            self.producer.produce(self.topic, outgoing_message)
        self.producer.poll(0)

    def publish(self, input_msg: dict, report: str):
        """Publish the report and other important info to Kafka.

        Default publish method, needs to be implemented in children classes.
        """
        raise NotImplementedError()

    def error(self, input_msg: dict, ex: Exception):
        """Handle pipeline errors by logging them."""
        log.warning("An error has ocurred during the processing of %s: %s", input_msg, ex)



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `idp` class."""

import os
from tempfile import NamedTemporaryFile
from unittest.mock import patch

import pytest

from ccx_messaging.downloaders.s3_downloader import S3Downloader


@patch("s3fs.S3FileSystem.open")
def test_download_existing_file(mock_s3_open):
    """Test downloading of existing file in s3."""
    downloader = S3Downloader(
        access_key="test",
        secret_key="test",
        bucket="testBucket",
        endpoint_url="https://s3.amazonaws.com",
    )

    test_file_name = ""

    with NamedTemporaryFile(delete=False) as test_file:
        test_file.write(b"Hello world")
        test_file_name = test_file.name

    with open(test_file_name, "rb") as fd:
        mock_s3_open.return_value.__enter__.return_value = fd
        with downloader.get("testfile") as path:
            assert os.path.exists(path)

    os.unlink(test_file_name)


@patch("s3fs.S3FileSystem.open")
def test_download_non_existing_file(mock_s3_open):
    """Test downloading of non existing file in s3."""
    downloader = S3Downloader(
        access_key="test",
        secret_key="test",
        bucket="testBucket",
        endpoint_url="https://s3.amazonaws.com",
    )

    mock_s3_open.side_effect = FileNotFoundError()

    with pytest.raises(FileNotFoundError):
        with downloader.get("non_existend_file"):
            pytest.fail()



"""Utility broker to improve Ssentry exception handling."""

import logging

from insights.core.dr import Broker
from insights.core.dr import MissingRequirements
from insights.core.spec_factory import ContentException

from sentry_sdk import capture_exception


class SentryMonitoredBroker(Broker):
    """Implementation of Broker with custom Sentry capturing logic."""

    def __init__(self, *args, **kwargs):
        """Initialize the SentryMonitoredBroker."""
        super().__init__(*args, **kwargs)
        self.logger = logging.getLogger(__name__)

    def add_exception(self, component, ex, tb=None):
        """Check added exception in order to use it with Sentry or not."""
        super().add_exception(component, ex, tb)

        # prevent MissingRequirements and ContentException from being sent to sentry
        if not isinstance(ex, (MissingRequirements, ContentException)):
            self.logger.debug("Sending exception to Sentry: %s", type(ex))
            capture_exception(ex)



# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `KafkaMultiplexorConsumer` class."""

import datetime
import json
from unittest.mock import MagicMock, patch

import pytest
from freezegun import freeze_time

from ccx_messaging.consumers.decoded_ingress_consumer import DecodedIngressConsumer
from ccx_messaging.error import CCXMessagingError

from . import KafkaMessage


_INVALID_TYPE_VALUES = [
    None,
    42,
    3.14,
    True,
    [],
    {},
]


@pytest.mark.parametrize("value", _INVALID_TYPE_VALUES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_type(mock_consumer, value):
    """Test that passing invalid data type to `deserialize` raises an exception."""
    sut = DecodedIngressConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(value)


@pytest.mark.parametrize("value", _INVALID_TYPE_VALUES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_type_as_kafka_message(mock_consumer, value):
    """Test that passing invalid data type to `deserialize` raises an exception."""
    sut = DecodedIngressConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(KafkaMessage(value))


_INVALID_MESSAGES = [
    "",
    "{}",
    '{"noturl":"https://s3.com/hash"}',
    '{"url":"value"',
    '"url":"value"}',
    '"url":"value"',
    '"{"url":"value"}"',
    # incorrect identity
    '{"url": "https://s3.com/hash", '
    '"identity": {"external": {"internal": {"org_id": "12345678"}}}, '
    '"timestamp": "2020-01-23T16:15:59.478901889Z"}',
    '{"url": "https://s3.com/hash", '
    '"identity": {"internal": {"internal": {"orgs": "12345678"}}}, '
    '"timestamp": "2020-01-23T16:15:59.478901889Z"}',
    # incorrect cluster_name
    '{"url": "https://s3.com/hash", '
    '"identity": {"identity": {"internal": {"org_id": "12345678"}}}, '
    '"timestamp": "2020-01-23T16:15:59.478901889Z", '
    '"cluster_name": 1}',
]


@pytest.mark.parametrize("msg", _INVALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_format_str(mock_consumer, msg):
    """Test that passing a malformed message to `deserialize` raises an exception."""
    sut = DecodedIngressConsumer(None, None, None, None)
    message = KafkaMessage(msg)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(message)


_VALID_MESSAGES = [
    '{"url": "https://s3.com/hash", '
    '"identity": {"identity": {"internal": {"org_id": "12345678"}}}, '
    '"timestamp": "2020-01-23T16:15:59.478901889Z"}',
    '{"url": "https://s3.com/hash", '
    '"identity": {"identity": {"internal": {"org_id": "12345678"}}}, '
    '"timestamp": "2020-01-23T16:15:59.478901889Z", '
    '"cluster_name": "c9d116ce-93db-4c19-abe3-0de1d3554f99"}',
    # null cluster
    '{"url": "https://s3.com/hash", '
    '"identity": {"identity": {"internal": {"org_id": "12345678"}}}, '
    '"timestamp": "2020-01-23T16:15:59.478901889Z", '
    '"cluster_name": null}',
]


@pytest.mark.parametrize("msg", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_str(msg):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = DecodedIngressConsumer(None, None, None, incoming_topic=None)
    out = json.loads(msg)
    if "cluster_name" not in msg:
        out["cluster_name"] = None
    assert sut.deserialize(KafkaMessage(msg)) == out


@pytest.mark.parametrize("msg", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_bytes(msg):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = DecodedIngressConsumer(None, None, None, incoming_topic=None)
    out = json.loads(msg)
    if "cluster_name" not in msg:
        out["cluster_name"] = None
    assert sut.deserialize(KafkaMessage(msg.encode())) == out


@pytest.mark.parametrize("msg", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_bytearray(msg):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = DecodedIngressConsumer(None, None, None, incoming_topic=None)
    out = json.loads(msg)
    if "cluster_name" not in msg:
        out["cluster_name"] = None
    assert sut.deserialize(KafkaMessage(bytearray(msg.encode()))) == out


@pytest.mark.parametrize("msg", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_handles_valid(msg):
    """Test that `handles` method returns True for valid messages."""
    sut = DecodedIngressConsumer(None, None, None, incoming_topic=None)
    assert sut.handles(KafkaMessage(msg))


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.handles", lambda *a, **k: True)
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.fire", lambda *a, **k: None)
def test_last_received_message_time_is_updated():
    """Check that the variable last_received_message_time used by a Thread is correctly updated.

    [CCXDEV-14812] the variable is never updated
    """
    t1 = datetime.datetime(2025, 1, 31, 12, 0, 0, tzinfo=datetime.timezone.utc)
    t2 = datetime.datetime(2025, 1, 31, 12, 20, 0, tzinfo=datetime.timezone.utc)

    with freeze_time(t1) as frozen_time:
        sut = DecodedIngressConsumer(None, None, None, incoming_topic=None)
        input_msg = KafkaMessage("{}")

        frozen_time.move_to(t2)

        with patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.process", lambda: None):
            sut.process_msg(input_msg)

        assert sut.last_received_message_time == t2.timestamp()


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_extract_cluster_id():
    """Check that the cluster_id is extracted from the identity field when not available.

    [CCXDEV-15056] Missing cluster id in some messages
    """
    sut = DecodedIngressConsumer(None, None, None, incoming_topic=None)
    cluster_id = "c9d116ce-93db-4c19-abe3-0de1d3554f99"
    msg = {
        "url": "https://s3.com/hash",
        "identity": {
            "identity": {
                "system": {
                    "cluster_id": cluster_id,
                },
                "internal": {
                    "org_id": "1234",
                },
            },
        },
        "timestamp": "2020-01-23T16:15:59.478901889Z",
    }

    assert sut.deserialize(KafkaMessage(json.dumps(msg)))["cluster_name"] == cluster_id



"""Kafka consumer implementation using Confluent Kafka library."""

import importlib.metadata
import logging
import time
from datetime import datetime
from threading import Thread

from confluent_kafka import (
    Consumer as ConfluentConsumer,
    KafkaException,
    Message,
    Producer,
    TIMESTAMP_NOT_AVAILABLE,
)
from insights.core.exceptions import InvalidContentType
from insights_messaging.consumers import Consumer

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.ingress import parse_ingress_message
from ccx_messaging.monitored_broker import SentryMonitoredBroker
from ccx_messaging.utils.kafka_config import kafka_producer_config_cleanup


LOG = logging.getLogger(__name__)
MAX_ELAPSED_TIME_BETWEEN_MESSAGES = 60 * 60


class KafkaConsumer(Consumer):
    """Consumer based in Confluent Kafka."""

    def __init__(
        self,
        publisher,
        downloader,
        engine,
        incoming_topic,
        kafka_broker_config=None,
        platform_service=None,
        dead_letter_queue_topic=None,
        max_record_age=7200,
        processing_timeout_s=0,
        **kwargs,
    ):
        """Initialise the KafkaConsumer object and related handlers."""
        requeuer = kwargs.pop("requeuer", None)
        super().__init__(publisher, downloader, engine, requeuer=requeuer)

        if kafka_broker_config:
            kwargs.update(kafka_broker_config)

        # Confluent initialization
        LOG.info(
            "Consuming topic '%s' from brokers %s as group '%s'",
            incoming_topic,
            kwargs.get("bootstrap.servers", None),
            kwargs.get("group.id", None),
        )

        LOG.debug("Confluent Kafka consumer extra configuration arguments: %s", kwargs)

        self.consumer = ConfluentConsumer(kwargs)
        self.consumer.subscribe([incoming_topic])

        # Self handled vars
        self.log_pattern = f"topic: {incoming_topic}, group.id: {kwargs.get('group.id', None)}"

        # Service to filter in messages
        self.platform_service = platform_service

        self.max_record_age = max_record_age
        self.last_received_message_time = time.time()
        self.check_elapsed_time_thread = Thread(
            target=self.check_last_message_received_time, daemon=True
        )
        self.check_elapsed_time_thread.start()

        self.processing_timeout = processing_timeout_s

        # DLQ
        self.dlq_producer = None
        self.dead_letter_queue_topic = dead_letter_queue_topic

        if self.dead_letter_queue_topic is not None:
            self.dlq_producer = Producer(kafka_producer_config_cleanup(kwargs))

        try:
            self.ocp_rules_version = importlib.metadata.version("ccx-rules-ocp")

        except importlib.metadata.PackageNotFoundError:
            logging.info("No OCP rules package is installed.")
            self.ocp_rules_version = None

    def get_url(self, input_msg: dict) -> str:
        """Retrieve URL to storage (S3/Minio) from Kafka message.

        Same as previous 2 methods, when we receive and figure out the
        message format, we can modify this method
        """
        try:
            url = input_msg["url"]
            LOG.debug(
                "Extracted URL from input message: %s",
                url,
            )
            return url

        # This should never happen, but let's check it just to be absolutely sure.
        # The `handles` method should prevent this from
        # being called if the input message format is wrong.
        except Exception as ex:
            LOG.warning("Unable to extract URL from input message: %s", ex)
            raise CCXMessagingError("Unable to extract URL from input message") from ex

    def run(self):
        """Consume message and proccess."""
        try:
            while True:
                received = self.consumer.consume(timeout=5.0)  # return empty if timeout

                for msg in received:
                    self.process_msg(msg)

        except KeyboardInterrupt:
            LOG.info("Cancelled by user")

        except KafkaException as ex:
            LOG.fatal("Fatal error: %s", ex)

        finally:
            LOG.info("Closing consumer")
            self.consumer.close()

    def handles(self, msg: Message) -> bool:
        """Check headers, format and other characteristics that can make the message unusable."""
        if self.platform_service:
            headers = msg.headers()
            if not headers:
                LOG.debug("Message filtered: no headers in message")
                return False

            headers = dict(headers)
            destination_service = headers.get("service", b"").decode()

            if destination_service != self.platform_service:
                LOG.debug("Message filtered: wrong destination service: %s", destination_service)
                self.fire("on_filter")
                return False

        return self._handles_timestamp_check(msg)

    def _handles_timestamp_check(self, msg: Message):
        """Check the timestamp of the msg."""
        if self.max_record_age == -1:
            return True

        timestamp_type, timestamp = msg.timestamp()

        if timestamp_type == TIMESTAMP_NOT_AVAILABLE:
            LOG.debug("Cannot check the incoming message timestamp.")
            return True

        # Kafka record timestamp is int64 in milliseconds.
        current = time.time()
        if (timestamp / 1000) < (current - self.max_record_age):
            LOG.debug("Skipping message due to its timestamp (too old)")
            return False

        return True

    def process_msg(self, msg: Message) -> None:
        """Process the message for the engine."""
        if not msg:
            LOG.debug("Empty record. Should not happen")
            return

        self.last_received_message_time = time.time()
        if msg.error():
            raise KafkaException(msg.error())

        if not self.handles(msg):
            # already logged in self.handles
            return

        try:
            if self.ocp_rules_version:
                LOG.debug("Processing message using OCP rules version %s", self.ocp_rules_version)
            # Deserialize
            value = self.deserialize(msg)

            # Core Messaging process
            self.process(value)

        except InvalidContentType as ex:
            LOG.warning("The archive cannot be processed by Insights: %s", ex)
            self.process_dead_letter(msg)

        except CCXMessagingError as ex:
            LOG.warning(
                "Unexpected error processing incoming message. (%s): %s. Error: %s",
                self.log_pattern,
                msg.value(),
                ex,
            )
            self.process_dead_letter(msg)

        except TimeoutError as ex:
            self.fire("on_process_timeout")
            LOG.exception(ex)
            self.process_dead_letter(msg)

        except Exception as ex:  # pylint: disable=broad-exception-caught
            LOG.exception(ex)
            self.process_dead_letter(msg)

    def deserialize(self, msg: Message) -> dict:
        """Deserialize the message received from Kafka into a dictionary."""
        if not msg:
            raise CCXMessagingError("No incoming message: %s", msg)

        try:
            value = msg.value()
        except AttributeError as ex:
            raise CCXMessagingError("Invalid incoming message type: %s", type(msg)) from ex

        LOG.debug("Deserializing incoming message(%s): %s", self.log_pattern, value)

        if not value:
            raise CCXMessagingError("Unable to read incoming message: %s", value)

        deserialized_msg = parse_ingress_message(value)

        if not deserialized_msg.get("cluster_name"):
            cluster_id = (
                deserialized_msg.get("identity", {})
                .get("identity", {})
                .get("system", {})
                .get("cluster_id", None)
            )
            deserialized_msg["cluster_name"] = cluster_id

        return deserialized_msg

    def check_last_message_received_time(self):
        """Verify elapsed time between received messages and warn if too long.

        Checks if the last received message was received more than one hour ago
        and sends an alert if it is the case
        """
        while True:
            if time.time() - self.last_received_message_time >= MAX_ELAPSED_TIME_BETWEEN_MESSAGES:
                LOG.warning("No new messages in the queue")
            # To do the minimum interruptions possible, sleep for one hour
            time.sleep(MAX_ELAPSED_TIME_BETWEEN_MESSAGES)

    def create_broker(self, input_msg):
        """Create a suitable `Broker` to be pass arguments to the `Engine`."""
        broker = SentryMonitoredBroker()

        # Some engines expect some data for its own usage, like the following:
        # (NOTE: The fields should be always present because of schema validation
        # done in the other method, default values are just a sanity check.)
        org_id = (
            input_msg.get("identity", {})
            .get("identity", {})
            .get("internal", {})
            .get("org_id", None)
        )
        date = datetime.fromisoformat(input_msg.get("timestamp", "0"))

        broker["org_id"] = org_id
        broker["cluster_id"] = input_msg["cluster_name"]
        broker["original_path"] = input_msg["url"]
        broker["year"] = f"{date.year:04}"
        broker["month"] = f"{date.month:02}"
        broker["day"] = f"{date.day:02}"
        broker["hour"] = f"{date.hour:02}"
        broker["minute"] = f"{date.minute:02}"
        broker["second"] = f"{date.second:02}"
        broker["time"] = f"{date.hour:02}{date.minute:02}{date.second:02}"

        return broker

    def process_dead_letter(self, msg: Message) -> None:
        """Send the message to a dead letter queue in a different Kafka topic."""
        if not self.dlq_producer:
            return

        LOG.info("Sending the message to dead letter queue topic")
        self.dlq_producer.produce(
            self.dead_letter_queue_topic,
            msg.value(),
        )



# Copyright 2024 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module that defines an Engine class for processing a downloaded archive."""

import logging
import os
from io import StringIO

from insights.core import dr
from insights.core.archives import extract
from insights.core.hydration import initialize_broker
from insights_messaging.engine import Engine as ICMEngine

log = logging.getLogger(__name__)


class OCPEngine(ICMEngine):
    """Extract, download and process IO archive."""

    def process(self, broker, path):
        """Get results from applying Insights rules to the downloaded archive.

        The archive is extracted and processed if the `openshift_lightspeed.json`
        file is not found within its root directory. The JSON resulting from applying
        the Insights rules is retrieved and returned as the method's output. Otherwise,
        None is returned.
        """
        for w in self.watchers:
            w.watch_broker(broker)

        try:
            self.fire("pre_extract", broker, path)
            with extract(
                path, timeout=self.extract_timeout, extract_dir=self.extract_tmp_dir
            ) as extraction:
                ctx, broker = initialize_broker(extraction.tmp_dir, broker=broker)
                self.fire("on_extract", ctx, broker, extraction)

                ols_file = os.path.join(extraction.tmp_dir, "openshift_lightspeed.json")
                if os.path.exists(ols_file):
                    log.debug("archive contains openshift_lightspeed.json file; skipping")
                    return "{}"

                output = StringIO()
                with self.Formatter(broker, stream=output):
                    dr.run_components(self.target_components, self.components_dict, broker=broker)
                output.seek(0)
                result = output.read()
                self.fire("on_engine_success", broker, result)
                return result
        except Exception as ex:
            self.fire("on_engine_failure", broker, ex)
            raise
        finally:
            self.fire("on_engine_complete", broker)



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module that defines an Engine class for processing a downloaded archive."""

import logging
import os.path

from insights.core.archives import extract
from insights.core.hydration import initialize_broker
from insights_messaging.engine import Engine as ICMEngine

log = logging.getLogger(__name__)


class SHAExtractorEngine(ICMEngine):
    """Engine for extraction of downloading tar archive and selecting a file to be processed."""

    def process(self, broker, path):
        """Retrieve SHA records from a downloaded archive.

        The archive is extracted and if the SHA records are found, the JSON is retrieved
        from the file and returned as the method output. Otherwise, None is returned.
        """
        for watcher in self.watchers:
            watcher.watch_broker(broker)

        try:
            self.fire("pre_extract", broker, path)

            with extract(
                path, timeout=self.extract_timeout, extract_dir=self.extract_tmp_dir
            ) as extraction:
                ctx, broker = initialize_broker(extraction.tmp_dir, broker=broker)

                self.fire("on_extract", ctx, broker, extraction)

                try:
                    filename = os.path.join(extraction.tmp_dir, "config", "workload_info.json")
                    with open(filename, encoding="utf-8") as stream:
                        result = stream.read()
                        log.debug("workload info found, starting publishing process")
                        self.fire("on_engine_success", broker, result)
                        return result
                except FileNotFoundError:
                    log.debug("archive does not contain workload info; skipping")
                    return None

        except Exception as ex:
            self.fire("on_engine_failure", broker, ex)
            raise
        finally:
            self.fire("on_engine_complete", broker)



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing JSON schemas used by the applications."""

# Schema of the input message consumed from Kafka.
INPUT_MESSAGE_SCHEMA = {
    "type": "object",
    "properties": {
        "url": {"type": "string"},
        "b64_identity": {"type": "string", "contentEncoding": "base64"},
        "timestamp": {"type": "string"},
    },
    "required": ["url", "b64_identity", "timestamp"],
}

IDENTITY_SCHEMA = {
    "type": "object",
    "properties": {
        "identity": {
            "type": "object",
            "properties": {
                "account_number": {"type": "string"},
                "auth_type": {"type": "string"},
                "internal": {
                    "type": "object",
                    "properties": {
                        "auth_time": {"type": "number"},
                        "org_id": {"type": "string"},
                    },
                    "required": ["org_id"],
                },
                "type": {"type": "string"},  # type is a property of the identity
                "user": {
                    "type": "object",
                    "properties": {
                        "email": {"type": "string"},
                        "first_name": {"type": "string"},
                        "is_active": {"type": "boolean"},
                        "is_internal": {"type": "boolean"},
                        "is_org_admin": {"type": "boolean"},
                        "last_name": {"type": "string"},
                        "locale": {"type": "string"},
                        "username": {"type": "string"},
                    },
                },
            },
            "required": ["internal"],
        },
    },
    "required": ["identity"],
}

ARCHIVE_SYNCED_SCHEMA = {
    "type": "object",
    "properties": {
        "path": {"type": "string"},
        "original_path": {"type": "string"},
        "metadata": {
            "type": "object",
            "properties": {
                "cluster_id": {"type": "string"},
                "external_organization": {"type": "string"},
            },
            "required": ["cluster_id"],
        },
    },
    "required": ["path", "metadata"],
}

RULES_RESULTS_SCHEMA = {
    "type": "object",
    "properties": {
        "path": {"type": "string"},
        "metadata": {
            "type": "object",
            "properties": {
                "cluster_id": {"type": "string"},
                "external_organization": {"type": "string"},
            },
            "required": ["cluster_id"],
        },
        "report": {"type": "object"},
    },
    "required": ["path", "metadata", "report"],
}



"""Utilities related to Ingress message format."""

import base64
import binascii
import json
import logging

import jsonschema

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.schemas import IDENTITY_SCHEMA, INPUT_MESSAGE_SCHEMA


LOG = logging.getLogger(__name__)


def parse_identity(encoded_identity: bytes) -> dict:
    """Generate a dictionary object from a base64 encoded input."""
    try:
        decoded_identity = base64.b64decode(encoded_identity)
        identity = json.loads(decoded_identity)

        jsonschema.validate(instance=identity, schema=IDENTITY_SCHEMA)
        return identity

    except TypeError as ex:
        LOG.warning("Bad argument type %s", encoded_identity)
        raise CCXMessagingError("Bad argument type") from ex

    except binascii.Error as ex:
        LOG.warning("Base64 encoded identity could not be parsed: %s", encoded_identity)
        raise CCXMessagingError("Base64 encoded identity could not be parsed") from ex

    except json.JSONDecodeError as ex:
        LOG.warning("Unable to decode received message: %s", decoded_identity)
        raise CCXMessagingError("Unable to decode received message") from ex

    except jsonschema.ValidationError as ex:
        LOG.warning("Invalid input message JSON schema: %s", identity)
        raise CCXMessagingError("Invalid input message JSON schema") from ex


def parse_ingress_message(message: bytes) -> dict:
    """Parse a bytes messages into a dictionary, decoding encoded values."""
    try:
        deserialized_message = json.loads(message)
        jsonschema.validate(instance=deserialized_message, schema=INPUT_MESSAGE_SCHEMA)

    except TypeError as ex:
        LOG.warning("Incorrect message type: %s", message)
        raise CCXMessagingError("Incorrect message type") from ex

    except json.JSONDecodeError as ex:
        LOG.warning("Unable to decode received message: %s", message)
        raise CCXMessagingError("Unable to decode received message") from ex

    except jsonschema.ValidationError as ex:
        LOG.warning("Invalid input message JSON schema: %s", deserialized_message)
        raise CCXMessagingError("Invalid input message JSON schema") from ex

    LOG.debug("JSON schema validated: %s", deserialized_message)

    encoded_identity = deserialized_message.pop("b64_identity")
    identity = parse_identity(encoded_identity)
    deserialized_message["identity"] = identity
    return deserialized_message



# Copyright 2023 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test for the ccx_messaging.utils.logging module."""

import logging
from unittest.mock import patch

import pytest
from watchtower import CloudWatchLogHandler

from ccx_messaging.utils.logging import get_mac_address, setup_watchtower


INVALID_INITIALIZATIONS = [
    {},
    {
        "LOGGING_TO_CW_ENABLED": "True",
    },
    {
        "LOGGING_TO_CW_ENABLED": "True",
        "CW_AWS_SECRET_ACCESS_KEY": "secret",
        "AWS_REGION_NAME": "aws-region1",
        "CW_LOG_GROUP": "log-group",
        "CW_STREAM_NAME": "stream",
    },
    {
        "LOGGING_TO_CW_ENABLED": "True",
        "CW_AWS_ACCESS_KEY_ID": "access_key",
        "AWS_REGION_NAME": "aws-region1",
        "CW_LOG_GROUP": "log-group",
        "CW_STREAM_NAME": "stream",
    },
    {
        "LOGGING_TO_CW_ENABLED": "True",
        "CW_AWS_ACCESS_KEY_ID": "access_key",
        "CW_AWS_SECRET_ACCESS_KEY": "secret",
        "CW_LOG_GROUP": "log-group",
        "CW_STREAM_NAME": "stream",
    },
    {
        "LOGGING_TO_CW_ENABLED": "True",
        "CW_AWS_ACCESS_KEY_ID": "access_key",
        "CW_AWS_SECRET_ACCESS_KEY": "secret",
        "AWS_REGION_NAME": "aws-region1",
        "CW_STREAM_NAME": "stream",
    },
    {
        "LOGGING_TO_CW_ENABLED": "True",
        "CW_AWS_ACCESS_KEY_ID": "access_key",
        "CW_AWS_SECRET_ACCESS_KEY": "secret",
        "AWS_REGION_NAME": "aws-region1",
        "CW_LOG_GROUP": "log-group",
    },
]


@pytest.mark.parametrize("invalid_environ", INVALID_INITIALIZATIONS)
def test_setup_watchtower_bad_configuration(invalid_environ):
    """Check that if no special env var is defined, cloudwatch logging is not configured."""
    root_logger = logging.getLogger()
    number_of_handlers = len(root_logger.handlers)

    with patch("os.environ", new=invalid_environ):
        setup_watchtower(None)
        assert number_of_handlers == len(root_logger.handlers)


# Set of valid environments to setup watchtower, no/info/wrong level (default to info)
VALID_INITIALIZATIONS = [
    {
        "LOGGING_TO_CW_ENABLED": "True",
        "CW_AWS_ACCESS_KEY_ID": "access_key",
        "CW_AWS_SECRET_ACCESS_KEY": "secret",
        "AWS_REGION_NAME": "aws-region1",
        "CW_LOG_GROUP": "log-group",
        "CW_STREAM_NAME": "stream",
    },
    {
        "LOGGING_TO_CW_ENABLED": "True",
        "CW_AWS_ACCESS_KEY_ID": "access_key",
        "CW_AWS_SECRET_ACCESS_KEY": "secret",
        "AWS_REGION_NAME": "aws-region1",
        "CW_LOG_GROUP": "log-group",
        "CW_STREAM_NAME": "stream",
        "CW_LOG_LEVEL": "INFO",
    },
    {
        "LOGGING_TO_CW_ENABLED": "True",
        "CW_AWS_ACCESS_KEY_ID": "access_key",
        "CW_AWS_SECRET_ACCESS_KEY": "secret",
        "AWS_REGION_NAME": "aws-region1",
        "CW_LOG_GROUP": "log-group",
        "CW_STREAM_NAME": "stream",
        "CW_LOG_LEVEL": "info",
    },
    {
        "LOGGING_TO_CW_ENABLED": "True",
        "CW_AWS_ACCESS_KEY_ID": "access_key",
        "CW_AWS_SECRET_ACCESS_KEY": "secret",
        "AWS_REGION_NAME": "aws-region1",
        "CW_LOG_GROUP": "log-group",
        "CW_STREAM_NAME": "stream",
        "CW_LOG_LEVEL": "NOT_A_LOG_LEVEL_AT_ALL",
    },
]


@pytest.mark.parametrize("environment", VALID_INITIALIZATIONS)
def test_setup_watchtower_info_level(environment):
    """Check that when a valid environment is defined, handler is added."""
    root_logger = logging.getLogger()
    number_of_handlers = len(root_logger.handlers)

    with patch("os.environ", new=environment):
        setup_watchtower(None)
        assert number_of_handlers + 1 == len(root_logger.handlers)
        for handler in root_logger.handlers:
            if isinstance(handler, CloudWatchLogHandler):
                cloudwatch_handler = handler
                break

        assert cloudwatch_handler.level == logging.INFO
        root_logger.removeHandler(cloudwatch_handler)


def test_get_mac_address_virtual():
    """Check that, if no real MAC is returned by the system, None is returned."""
    with patch("ccx_messaging.utils.logging.uuid.getnode") as getnode_mock:
        getnode_mock.return_value = 68039291015483  # magic number that does the trick
        assert get_mac_address() is None


def test_get_mac_address_real():
    """Check that, if no real MAC is returned by the system, None is returned."""
    with patch("ccx_messaging.utils.logging.uuid.getnode") as getnode_mock:
        getnode_mock.return_value = 66939779387710  # magic number that does the trick
        assert get_mac_address() == "3c:e1:a1:c5:91:3e"



# Copyright 2023 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module that implements a Confluent based Kafka publisher to send the workload info."""

import json
import logging
from json import JSONDecodeError

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.publishers.kafka_publisher import KafkaPublisher


log = logging.getLogger(__name__)


class WorkloadInfoPublisher(KafkaPublisher):
    """WorkloadInfoPublisher based on Confluent Kafka Producer.

    The workload info is received from a custom engine as a JSON string.
    The response, among other information extracted from the consumer's received
    message, is sent over a Kafka topic to be consumed by other services.
    """

    def __init__(self, outgoing_topic, kafka_broker_config=None, **kwargs):
        """Construct a new `RuleProcessingPublisher` given `kwargs` from the config YAML."""
        super().__init__(outgoing_topic, kafka_broker_config, **kwargs)
        self.outdata_schema_version = 2

    def publish(self, input_msg: dict, response: str | None):
        """Publish an EOL-terminated JSON message to the output Kafka topic.

        The input_msg contains content of message read from incoming Kafka
        topic. Such message should contains account info, cluster ID etc.

        The response is assumed to be a string representing a valid JSON object
        (it is read from file config/workload_info.json).

        Outgoing message is constructed by joining input_msg with response.

        A newline character will be appended to it, it will be converted into
        a byte array using UTF-8 encoding and the result of that will be sent
        to the producer to produce a message in the output Kafka topic.
        """
        try:
            org_id = int(input_msg["identity"]["identity"]["internal"]["org_id"])
        except (ValueError, TypeError, KeyError) as err:
            log.warning("Error extracting the OrgID: %s", err)
            raise CCXMessagingError("Error extracting the OrgID") from err

        try:
            account_number = int(input_msg["identity"]["identity"]["account_number"])
        except (ValueError, KeyError, TypeError) as err:
            log.warning("Error extracting the Account number: %s", err)
            account_number = None

        # outgoing message in form of JSON
        message = ""

        if response is None:
            log.debug("No response was generated for this archive. Skipping")
            return

        try:
            msg_timestamp = input_msg["timestamp"]
            output_msg = {
                "OrgID": org_id,
                "ClusterName": input_msg["cluster_name"],
                "Images": json.loads(response),
                "LastChecked": msg_timestamp,
                "Version": self.outdata_schema_version,
                "RequestId": input_msg.get("request_id"),
            }

            if account_number is not None:
                output_msg["AccountNumber"] = account_number

            # convert dictionary to JSON (string)
            message = json.dumps(output_msg) + "\n"

            log.debug("Sending response to the %s topic.", self.topic)

            # Convert message string into a byte array.
            self.produce(message.encode("utf-8"))
            log.debug("Message has been sent successfully.")
            log.debug(
                "Message context: OrgId=%s, AccountNumber=%s, "
                'ClusterName="%s", NumImages: %d, LastChecked="%s, Version=%d"',
                output_msg["OrgID"],
                output_msg.get("AccountNumber", ""),
                output_msg["ClusterName"],
                len(output_msg["Images"]),
                output_msg["LastChecked"],
                output_msg["Version"],
            )

        except (KeyError, TypeError, UnicodeEncodeError, JSONDecodeError) as err:
            log.warning("Error encoding the response to publish: %s", message)
            raise CCXMessagingError("Error encoding the response to publish") from err



# Copyright 2024 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""S3 Engine Class and related functions."""

import json
import logging
import os
import tarfile

import sentry_sdk
from insights_messaging.engine import Engine

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.utils.s3_uploader import S3Uploader
from ccx_messaging.utils.sliced_template import SlicedTemplate


LOG = logging.getLogger(__name__)


def create_metadata(components: dict[str, str]):
    """Create a metadata for Publishing."""
    msg_metadata = {
        "cluster_id": components.get("cluster_id"),
        "external_organization": components.get("org_id"),
    }
    LOG.debug("msg_metadata %s", msg_metadata)
    return msg_metadata


class S3UploadEngine(Engine):
    """Engine for processing the metadata and ceph path of downloaded archive and uploading it to ceph bucket."""  # noqa: E501

    def __init__(
        self,
        formatter,
        target_components=None,
        extract_timeout=None,
        unpacked_archive_size_limit=None,
        extract_tmp_dir=None,
        dest_bucket=None,
        access_key=None,
        secret_key=None,
        endpoint=None,
        archives_path_prefix=None,
        archive_name_pattern="$cluster_id[:2]/$cluster_id/$year$month/$day/$time.tar.gz",
        s3_acl_enabled=True,
    ):
        """Initialize engine for S3 upload.

        The S3 server is specified by the `endpoint` argument, and its credentials using
        `access_key` and `secret_key` arguments.

        The bucket where the archives will be uploaded is specified by `dest_bucket` argument.

        A common prefix for all the archives uploaded can be defined using `archives_path_prefix`,
        that should be a `str` without the starting /.

        `archive_name_pattern` will define a string template. The following substitutions are
        available:
          - `cluster_id`: it will be replaced by the cluster ID for the processed archive.
          - `timestamp`, `year`, `month`, `day` and `time`: those will be extracted from incoming
            archive path or other availables timestamps.
          - `archive`: it will be replaced by the base name of the archive.
        """
        super().__init__(
            formatter=formatter,
            target_components=target_components,
            extract_timeout=extract_timeout,
            unpacked_archive_size_limit=unpacked_archive_size_limit,
            extract_tmp_dir=extract_tmp_dir,
        )
        self.dest_bucket = dest_bucket
        self.archives_path_prefix = archives_path_prefix
        self.archive_name_template = SlicedTemplate(archive_name_pattern)
        self.uploader = S3Uploader(
            access_key=access_key,
            secret_key=secret_key,
            endpoint=endpoint,
            acl_enabled=s3_acl_enabled,
        )

    def process(self, broker, local_path):
        """Create metadata and target_path from downloaded archive and uploads it to ceph bucket."""
        LOG.debug("Processing %s for uploading", local_path)
        LOG.debug("Broker content: %s", broker.items())
        self.fire("pre_extract", broker, local_path)

        for w in self.watchers:
            w.watch_broker(broker)

        if broker["cluster_id"] is None:
            del broker["cluster_id"]
            broker["cluster_id"] = extract_cluster_id(local_path)

        target_path = self.compute_target_path(broker)
        LOG.debug(f"Uploading archive '{local_path}' as {self.dest_bucket}/{target_path}")
        self.uploader.upload_file(local_path, self.dest_bucket, target_path)
        LOG.debug(f"Uploaded archive '{local_path}' as {self.dest_bucket}/{target_path}")

        metadata = create_metadata(broker)
        report = {
            "path": target_path,
            "original_path": broker.get("original_path", ""),
            "metadata": metadata,
        }

        LOG.debug("Generated report: %s", report)
        self.fire("on_engine_success", broker, report)

        return json.dumps(report)

    def compute_target_path(self, components: dict[str, str]) -> str:
        """Compute S3 target path from the found archive name components.

        Target path is defined by the archive pattern.
        """
        path = self.archive_name_template.safe_substitute(components)

        if self.archives_path_prefix:
            return f"{self.archives_path_prefix}/{path}"

        else:
            return path


def extract_cluster_id(tar_path: str) -> str:
    """Check the content of the file in `tar_path` and extract the cluster_id."""
    ID_PATH = os.path.join("config", "id")
    LOG.debug("Looking for %s in file %s", ID_PATH, tar_path)

    try:
        with tarfile.open(tar_path) as tf:
            with tf.extractfile(ID_PATH) as id_file:
                return id_file.read().decode()

    except KeyError as ex:
        sentry_sdk.set_context(
            "archive_processing",
            {"tar_path": tar_path, "error_type": "missing_cluster_id", "id_path": ID_PATH},
        )
        raise CCXMessagingError("Archive doesn't contain cluster id") from ex

    except tarfile.ReadError as ex:
        sentry_sdk.set_context(
            "archive_processing", {"tar_path": tar_path, "error_type": "invalid_tarfile"}
        )
        raise CCXMessagingError("File doesn't look as a tarfile") from ex



"""Handlers for CLI commands and some utility functions."""

import argparse
import importlib.metadata
import logging
import os
import sys

from app_common_python import isClowderEnabled
from insights_messaging.appbuilder import AppBuilder

from ccx_messaging.utils.clowder import apply_clowder_config
from ccx_messaging.utils.logging import setup_watchtower
from ccx_messaging.utils.sentry import init_sentry


def parse_args() -> argparse.Namespace:
    """Parse the command line options and arguments."""
    parser = argparse.ArgumentParser()
    parser.add_argument("config", nargs="?", help="Application configuration.")
    parser.add_argument("--version", help="Show version.", action="store_true")
    return parser.parse_args()


def print_version() -> None:
    """Log version information."""
    logger = logging.getLogger(__name__)
    logger.info(
        "Python interpreter version: %d.%d.%d",
        sys.version_info.major,
        sys.version_info.minor,
        sys.version_info.micro,
    )
    logger.info(
        "%s version: %s",
        sys.argv[0],
        importlib.metadata.version("ccx-messaging"),
    )
    try:
        ocp_rules_version = importlib.metadata.version("ccx-rules-ocp")
        logger.info("ccx-rules-ocp version: %s", ocp_rules_version)

    except importlib.metadata.PackageNotFoundError:
        pass


def apply_config(config) -> int:
    """Apply configuration file provided as argument and run consumer."""
    with open(config) as file_:
        if isClowderEnabled() and os.getenv("CLOWDER_ENABLED") in ["True", "true", "1", "yes"]:
            manifest = apply_clowder_config(file_.read())
        else:
            manifest = file_.read()

        app_builder = AppBuilder(manifest)
        logging_config = app_builder.service["logging"]
        logging.config.dictConfig(logging_config)
        print_version()
        try:
            consumer = app_builder.build_app()
            setup_watchtower(logging_config)
            consumer.run()
            return 0

        except ModuleNotFoundError as ex:
            logging.error("Module not found: %s. Did you miss some dependency?", ex.name)
            return 1


def ccx_messaging() -> None:
    """Handle the ccx-messaging CLI command."""
    args = parse_args()

    if args.version:
        logging.basicConfig(format="%(message)s", level=logging.INFO)
        print_version()
        sys.exit(0)

    init_sentry(
        os.environ.get("SENTRY_DSN", None),
        None,
        os.environ.get("SENTRY_ENVIRONMENT", None),
        os.environ.get("SENTRY_ENABLED", "false").lower() == "true",
    )

    if args.config:
        retval = apply_config(args.config)
        sys.exit(retval)

    logger = logging.getLogger(__name__)
    logger.error(
        "Application configuration not provided. \
        Use 'ccx-data-pipeline <config>' to run the application",
    )
    sys.exit(1)



"""Module containing the consumer for the Kafka topic produced by the Archive Sync service."""

import logging
import time
from typing import Any

from confluent_kafka import Message, KafkaException
from insights.core.exceptions import InvalidContentType

from ccx_messaging.consumers.kafka_consumer import KafkaConsumer
from ccx_messaging.internal_pipeline import parse_archive_sync_msg
from ccx_messaging.error import CCXMessagingError
from ccx_messaging.monitored_broker import SentryMonitoredBroker


LOG = logging.getLogger(__name__)


class SyncedArchiveConsumer(KafkaConsumer):
    """Consumer for the topic produced by `synced_archive_publisher.SyncedArchivePublisher`."""

    def get_url(self, input_msg: dict[str, str]) -> str:
        """Retrieve path to the archive in the S3 storage from Kafka message."""
        # it's safe to asume the "path" is there because the message format is validated
        return input_msg["path"]

    def process_msg(self, msg: Message) -> None:
        """Process a single message received from the topic."""
        if not msg:
            LOG.debug("Empty record. Should not happen")
            return

        self.last_received_message_time = time.time()  # Base class thread control
        if msg.error():
            raise KafkaException(msg.error())

        try:
            # Deserialize
            value = self.deserialize(msg)

            if self.ocp_rules_version:
                LOG.debug("Processing message using OCP rules version %s", self.ocp_rules_version)

            # Core Messaging process
            self.process(value)

        except InvalidContentType as ex:
            LOG.warning("The archive cannot be processed by Insights: %s", ex)
            self.process_dead_letter(msg)

        except CCXMessagingError as ex:
            LOG.warning(
                "Unexpected error processing incoming message. (%s): %s. Error: %s",
                self.log_pattern,
                msg.value(),
                ex,
            )
            self.process_dead_letter(msg)

        except TimeoutError as ex:
            self.fire("on_process_timeout")
            LOG.warning("Process timeout: %s", ex)
            self.process_dead_letter(msg)

        except Exception as ex:  # pylint: disable=broad-exception-caught
            LOG.exception(ex)
            self.process_dead_letter(msg)

    def deserialize(self, msg: Message) -> dict:
        """Deserialize the message received from Kafka into a dictionary."""
        if not msg:
            raise CCXMessagingError("No incoming message: %s", msg)

        try:
            value = msg.value()
        except AttributeError as ex:
            raise CCXMessagingError("Invalid incoming message type: %s", type(msg)) from ex

        LOG.debug("Deserializing incoming message(%s): %s", self.log_pattern, value)

        if not value:
            raise CCXMessagingError("Unable to read incoming message: %s", value)

        deseralized_msg = parse_archive_sync_msg(value)
        LOG.debug("JSON message deserialized (%s): %s", self.log_pattern, deseralized_msg)
        return deseralized_msg

    def create_broker(self, input_msg: dict[str, Any]) -> SentryMonitoredBroker:
        """Create a suitable `Broker`."""
        return SentryMonitoredBroker()



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `KafkaConsumer` class."""

import datetime
import json
from unittest.mock import MagicMock, patch

import pytest
from freezegun import freeze_time

from confluent_kafka import KafkaException
from insights.core.exceptions import InvalidContentType

from ccx_messaging.consumers.synced_archive_consumer import SyncedArchiveConsumer
from ccx_messaging.error import CCXMessagingError

from . import KafkaMessage


# _REGEX_BAD_SCHEMA = r"^Unable to extract URL from input message: "
_INVALID_TYPE_VALUES = [
    None,
    42,
    3.14,
    True,
    [],
    {},
]


@pytest.mark.parametrize("value", _INVALID_TYPE_VALUES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_type(mock_consumer, value):
    """Test that passing invalid data type to `deserialize` raises an exception."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(value)


@pytest.mark.parametrize("value", _INVALID_TYPE_VALUES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_type_as_kafka_message(mock_consumer, value):
    """Test that passing invalid data type to `deserialize` raises an exception."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(KafkaMessage(value))


_VALID_MESSAGES = [
    (
        '{"path": "", "metadata": {"cluster_id": ""}}',
        {
            "path": "",
            "metadata": {
                "cluster_id": "",
            },
        },
    ),
    (
        '{"path": "path/to/archive.tgz", "metadata": {"cluster_id": ""}}',
        {
            "path": "path/to/archive.tgz",
            "metadata": {
                "cluster_id": "",
            },
        },
    ),
    (
        '{"path": "path/to/archive.tgz", "metadata": {"cluster_id": "a_cluster_name"},'
        '"original_path": ""}',
        {
            "path": "path/to/archive.tgz",
            "metadata": {
                "cluster_id": "a_cluster_name",
            },
            "original_path": "",
        },
    ),
    (
        '{"path": "path/to/archive.tgz", "original_path": "other/path/archive.tgz", '
        '"metadata": {"cluster_id": "a_cluster_name", "external_organization": ""}}',
        {
            "path": "path/to/archive.tgz",
            "metadata": {
                "cluster_id": "a_cluster_name",
                "external_organization": "",
            },
            "original_path": "other/path/archive.tgz",
        },
    ),
]


@pytest.mark.parametrize("msg,value", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_str(msg, value):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    assert sut.deserialize(KafkaMessage(msg)) == value


@pytest.mark.parametrize("msg,value", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_bytes(msg, value):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    assert sut.deserialize(KafkaMessage(msg.encode())) == value


@pytest.mark.parametrize("msg,value", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_bytearray(msg, value):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    assert sut.deserialize(KafkaMessage(bytearray(msg.encode()))) == value


_INVALID_MESSAGES = [
    "",
    '"path": "value"',
]


@pytest.mark.parametrize("msg", _INVALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_invalid_str(msg):
    """Test that invalid string JSON is not correctly deserialized."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(KafkaMessage(msg))


@pytest.mark.parametrize("msg, _", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_handles_valid(msg, _):
    """Test that `handles` method returns True for valid messages."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    assert sut.handles(KafkaMessage(msg))


# This would have been a valid input, but it's supposed to be a `dict`, not `str`.
_DICT_STR = '{"path": "bucket/file"}'

_INVALID_RECORD_VALUES = [
    "",
    _DICT_STR.encode("utf-8"),
    bytearray(_DICT_STR.encode("utf-8")),
    [],
]

_VALID_RECORD_VALUES = [
    {"path": ""},
    {"path": "bucket/file"},
    {"path": "https://a-valid-domain.com/precious_url"},
]


@pytest.mark.parametrize("value", _INVALID_RECORD_VALUES)
def test_get_url_invalid(value):
    """Test that `SyncedArchiveConsumer.get_url` raises the appropriate exception."""
    with pytest.raises(Exception):
        SyncedArchiveConsumer.get_url(None, value)


@pytest.mark.parametrize("value", _VALID_RECORD_VALUES)
def test_get_url_valid(value):
    """Test that `SyncedArchiveConsumer.get_url` returns the expected value."""
    assert SyncedArchiveConsumer.get_url(None, value) == value["path"]


def test_create_broker():
    """Test that `SyncedArchiveConsumer.create_broker` doesn't report any error."""
    assert SyncedArchiveConsumer.create_broker(None, {}) is not None


_VALID_KAFKA_MESSAGES = [KafkaMessage(value) for value in _VALID_RECORD_VALUES]
_VALID_KAFKA_MESSAGES.extend(
    [
        None,
    ]
)


@pytest.mark.parametrize("value", _VALID_KAFKA_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_process_msg(value: dict[str, str]):
    """Test right `process_msg` behaviour."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    with patch(
        "ccx_messaging.consumers.synced_archive_consumer.SyncedArchiveConsumer.process",
        lambda: None,
    ):
        sut.process_msg(value)


_INVALID_KAFKA_MESSAGES = [KafkaMessage(value) for value in _INVALID_RECORD_VALUES]


@pytest.mark.parametrize("value", _INVALID_KAFKA_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_process_invalid_msg(value: dict[str, str]):
    """Test right `process_msg` behaviour."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    with patch(
        "ccx_messaging.consumers.synced_archive_consumer.SyncedArchiveConsumer.process_dead_letter",
    ) as dlq_mock:
        sut.process_msg(value)
        assert dlq_mock.called


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_process_msg_raise_error():
    """Test when the `process` message raises an error."""
    sut = SyncedArchiveConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(KafkaException):
        sut.process_msg(KafkaMessage(error=True))


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.handles", lambda *a, **k: True)
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.fire", lambda *a, **k: None)
def test_last_received_message_time_is_updated():
    """Check that the variable last_received_message_time used by a Thread is correctly updated.

    [CCXDEV-14812] the variable is never updated
    """
    t1 = datetime.datetime(2025, 1, 31, 12, 0, 0, tzinfo=datetime.timezone.utc)
    t2 = datetime.datetime(2025, 1, 31, 12, 20, 0, tzinfo=datetime.timezone.utc)

    with freeze_time(t1) as frozen_time:
        sut = SyncedArchiveConsumer(None, None, None, None)
        input_msg = KafkaMessage("{}")

        frozen_time.move_to(t2)

        with patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.process", lambda: None):
            sut.process_msg(input_msg)

        assert sut.last_received_message_time == t2.timestamp()


_EXPECTED_EXCEPTIONS = [
    InvalidContentType(content_type="application/json"),
    CCXMessagingError("Test exception"),
    TimeoutError("Test timeout"),
    Exception("Test exception"),
]


@pytest.mark.parametrize("exception", _EXPECTED_EXCEPTIONS)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch("ccx_messaging.consumers.synced_archive_consumer.SyncedArchiveConsumer.deserialize")
def test_process_to_dlq(deserialize_mock, exception):
    """Test that the message is processed to DLQ when an error is raised."""
    deserialize_mock.side_effect = exception
    sut = SyncedArchiveConsumer(None, None, None, None)

    with patch(
        "ccx_messaging.consumers.synced_archive_consumer.SyncedArchiveConsumer.process_dead_letter",
    ) as dlq_mock:
        sut.process_msg(KafkaMessage(json.dumps(_VALID_RECORD_VALUES[0])))
        assert dlq_mock.called



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module including a mixed watcher to retrieve the cluster ID from the extracted file."""

import logging
import os
from uuid import UUID

from insights_messaging.watchers import ConsumerWatcher, EngineWatcher


LOG = logging.getLogger(__name__)


class ClusterIdWatcher(EngineWatcher, ConsumerWatcher):
    """Mixed `Watcher` that is able to watch both `Consumer` and `Engine`."""

    CLUSTER_ID_LENGTH = 36

    def __init__(self):
        """Initialize a `ClusterIdWatcher`."""
        self.last_record = None

    def on_recv(self, input_msg):
        """Get and stores the `ConsumerRecord` when a new Kafka record arrives."""
        self.last_record = input_msg

    def on_extract(self, ctx, broker, extraction):
        """Receive the notification when the file is extracted.

        The method get the directory where the files are extracted and find the
        id in the expected path.
        """
        if self.last_record is None:
            LOG.warning(
                "Unexpected data flow: watched extraction event without a previous receiving event"
            )
            return

        if self.last_record.get("cluster_name", None) is not None:
            return

        id_file_path = os.path.join(extraction.tmp_dir, "config", "id")

        try:
            with open(id_file_path) as id_file:
                cluster_uuid = id_file.read(ClusterIdWatcher.CLUSTER_ID_LENGTH)

                try:
                    UUID(cluster_uuid)
                    self.last_record["cluster_name"] = cluster_uuid

                except ValueError:
                    self.last_record["cluster_name"] = None
                    LOG.warning("The cluster id is not an UUID. Skipping its extraction")

        except FileNotFoundError:
            self.last_record["cluster_name"] = None
            LOG.warning(
                "The archive doesn't contain a valid Cluster Id file. Skipping its extraction"
            )

        except OSError:
            self.last_record["cluster_name"] = None
            LOG.warning(f"Could not read file: {id_file_path}")

        finally:
            self.last_record = None
